"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchInfosElement = void 0;
const core_1 = require("@ocsjs/core");
const easy_us_1 = require("easy-us");
/**
 * 判断是否有图片链接，如果有则使用 <img> 标签包裹，但如果已经被 <img> 包裹则不处理
 */
const transformImgLinkOfQuestion = (question) => {
    // 防止题目中包含 img 标签元素，所以先统一吧 img 标签替换成链接
    const dom = new DOMParser().parseFromString(question, 'text/html');
    for (const img of Array.from(dom.querySelectorAll('img'))) {
        img.replaceWith(img.src);
    }
    // 最后将所有图片链接替换成 img 标签
    return dom.documentElement.innerText.replace(/https?:\/\/.+?\.(png|jpg|jpeg|gif)/g, (img) => {
        return `<img src="${img}" />`;
    });
};
/**
 * 搜索结果元素
 */
class SearchInfosElement extends HTMLElement {
    constructor() {
        super(...arguments);
        /** 搜索结果 [题目，答案] */
        this.infos = [];
        /** 当前的题目 */
        this.question = '';
    }
    connectedCallback() {
        console.log('connectedCallback : ', this);
        const question = transformImgLinkOfQuestion(this.question || '无');
        this.append((0, easy_us_1.h)('div', [(0, easy_us_1.h)('span', { innerHTML: question }), (0, core_1.createQuestionTitleExtra)(this.question)], (div) => {
            div.style.padding = '4px';
        }), (0, easy_us_1.h)('hr'));
        this.append(...this.infos.map((info) => {
            return (0, easy_us_1.h)('details', { open: true }, [
                (0, easy_us_1.h)('summary', [(0, easy_us_1.h)('a', { href: info.homepage, innerText: info.name, target: '_blank' })]),
                ...(info.error
                    ? /** 显示错误信息 */
                        [(0, easy_us_1.h)('span', { className: 'error' }, [info.error || '网络错误或者未知错误'])]
                    : /** 显示结果列表 */
                        []).concat([
                    ...info.results.map((ans) => {
                        const title = transformImgLinkOfQuestion(ans[0] || this.question || '无');
                        const answer = transformImgLinkOfQuestion(ans[1] || '无');
                        return (0, easy_us_1.h)('div', { className: 'search-result' }, [
                            /** 题目 */
                            (0, easy_us_1.h)('div', { className: 'question' }, [(0, easy_us_1.h)('span', { innerHTML: title })]),
                            /** 答案 */
                            (0, easy_us_1.h)('div', { className: 'answer' }, [
                                (0, easy_us_1.h)('span', '答案：'),
                                ...(0, core_1.splitAnswer)(answer).map((a) => (0, easy_us_1.h)('code', { innerHTML: a }))
                            ])
                        ]);
                    })
                ])
            ]);
        }));
        core_1.$.onresize(this, (sr) => {
            sr.style.maxHeight = window.innerHeight / 2 + 'px';
        });
    }
}
exports.SearchInfosElement = SearchInfosElement;
