import { Project, Script } from 'easy-us';
export type LogType = 'log' | 'info' | 'debug' | 'warn' | 'error';
type RequestList = {
    id: string;
    url: string;
    method: string;
    type: string;
    data: any;
    headers: any;
    response?: string;
    error?: string;
    time: number;
}[];
/** 后台进程，处理与PC软件端的通讯，以及其他后台操作 */
export declare const BackgroundProject: Project<{
    elementRegister: Script<import("easy-us").ScriptConfigs, import("easy-us").ScriptMethods>;
    console: Script<{
        logs: {
            defaultValue: {
                type: LogType;
                content: string;
                time: number;
                stack: string;
            }[];
        };
    }, import("easy-us").ScriptMethods>;
    appConfigSync: Script<{
        notes: {
            defaultValue: string;
        };
        sync: {
            defaultValue: boolean;
        };
        connected: {
            defaultValue: boolean;
        };
        closeSync: {
            defaultValue: boolean;
            label: string;
            attrs: {
                type: string;
            };
        };
    }, import("easy-us").ScriptMethods>;
    update: Script<{
        notes: {
            defaultValue: string;
        };
        autoNotify: {
            defaultValue: boolean;
            label: string;
            attrs: {
                type: string;
                title: string;
            };
        };
        notToday: {
            defaultValue: number;
        };
        ignoreVersions: {
            defaultValue: string[];
        };
    }, {
        getLastVersion: () => Promise<{
            'last-version': string;
            resource: Record<string, string>;
            notes: string[];
        }>;
    }>;
    dev: Script<{
        notes: {
            defaultValue: string;
        };
    }, import("easy-us").ScriptMethods>;
    appLoginHelper: Script<import("easy-us").ScriptConfigs, import("easy-us").ScriptMethods>;
    errorHandle: Script<import("easy-us").ScriptConfigs, import("easy-us").ScriptMethods>;
    requestList: Script<{
        notes: {
            defaultValue: string;
        };
        enable: {
            label: string;
            attrs: {
                type: string;
            };
            defaultValue: boolean;
        };
        methodFilter: {
            label: string;
            tag: "select";
            attrs: {
                placeholder: string;
            };
            options: string[][];
            defaultValue: string;
        };
        typeFilter: {
            label: string;
            tag: "select";
            attrs: {
                placeholder: string;
            };
            options: string[][];
            defaultValue: string;
        };
        searchValue: {
            label: string;
            attrs: {
                placeholder: string;
            };
            defaultValue: string;
        };
        list: {
            defaultValue: RequestList;
        };
    }, {
        render: (list: RequestList) => void;
    }>;
}>;
type Console = Record<LogType, (...msg: any[]) => void>;
/** 日志对象，存储日志并显示在日志面板 */
export declare const $console: Console;
export {};
