"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.$console = exports.BackgroundProject = void 0;
const core_1 = require("@ocsjs/core");
const easy_us_1 = require("easy-us");
const gt_1 = __importDefault(require("semver/functions/gt"));
const common_1 = require("./common");
const __1 = require("..");
const render_1 = require("../render");
const search_infos_1 = require("../elements/search.infos");
const state = {
    console: {
        listenerIds: {
            logs: 0
        }
    },
    app: {
        listenerIds: {
            sync: 0,
            connected: 0,
            closeSync: 0
        }
    }
};
/** 后台进程，处理与PC软件端的通讯，以及其他后台操作 */
exports.BackgroundProject = easy_us_1.Project.create({
    name: '后台',
    domains: [],
    scripts: {
        elementRegister: new easy_us_1.Script({
            name: '🔗 元素注册',
            hideInPanel: true,
            matches: [['所有页面', /.*/]],
            onstart() {
                // 注册自定义元素
                easy_us_1.$.loadCustomElements([search_infos_1.SearchInfosElement]);
            }
        }),
        console: new easy_us_1.Script({
            name: '📄 日志输出',
            matches: [['所有', /.*/]],
            namespace: 'render.console',
            configs: {
                logs: {
                    defaultValue: []
                }
            },
            onrender({ panel }) {
                const getTypeDesc = (type) => type === 'info'
                    ? '信息'
                    : type === 'error'
                        ? '错误'
                        : type === 'warn'
                            ? '警告'
                            : type === 'debug'
                                ? '调试'
                                : '日志';
                const createLog = (log) => {
                    const date = new Date(log.time);
                    const item = (0, easy_us_1.h)('div', {
                        title: '双击复制日志信息',
                        className: 'item'
                    }, [
                        (0, easy_us_1.h)('span', { className: 'time' }, `${date.getHours().toFixed(0).padStart(2, '0')}:${date.getMinutes().toFixed(0).padStart(2, '0')} `),
                        (0, easy_us_1.h)('span', { className: log.type }, `[${getTypeDesc(log.type)}]`),
                        (0, easy_us_1.h)('span', ':' + log.content)
                    ]);
                    item.addEventListener('dblclick', () => {
                        navigator.clipboard.writeText(Object.keys(log)
                            .map((k) => `${k}: ${log[k]}`)
                            .join('\n'));
                    });
                    return item;
                };
                const showLogs = () => {
                    const div = (0, easy_us_1.h)('div', { className: 'card console' });
                    const logs = this.cfg.logs.map((log) => createLog(log));
                    if (logs.length) {
                        div.replaceChildren(...logs);
                    }
                    else {
                        div.replaceChildren((0, easy_us_1.h)('div', '暂无任何日志', (div) => {
                            div.style.textAlign = 'center';
                        }));
                    }
                    return { div, logs };
                };
                /**
                 * 判断滚动条是否滚到底部
                 */
                const isScrollBottom = (div) => {
                    const { scrollHeight, scrollTop, clientHeight } = div;
                    return scrollTop + clientHeight + 50 > scrollHeight;
                };
                const { div, logs } = showLogs();
                this.offConfigChange(state.console.listenerIds.logs);
                state.console.listenerIds.logs = this.onConfigChange('logs', (logs) => {
                    const log = createLog(logs[logs.length - 1]);
                    div.append(log);
                    setTimeout(() => {
                        if (isScrollBottom(div)) {
                            log.scrollIntoView();
                        }
                    }, 10);
                });
                const show = () => {
                    panel.body.replaceChildren(div);
                    setTimeout(() => {
                        var _a;
                        (_a = logs[logs.length - 1]) === null || _a === void 0 ? void 0 : _a.scrollIntoView();
                    }, 10);
                };
                show();
            }
        }),
        appConfigSync: new easy_us_1.Script({
            name: '🔄️ 软件配置同步',
            namespace: 'background.app',
            matches: [['所有页面', /./]],
            // 如果是在OCS软件中则不显示此页面
            hideInPanel: easy_us_1.$gm.getInfos() === undefined,
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        [
                            (0, easy_us_1.h)('span', [
                                '如果您使用',
                                (0, easy_us_1.h)('a', { href: 'https://docs.ocsjs.com/docs/app', target: '_blank' }, 'OCS桌面软件'),
                                '启动浏览器，并使用此脚本，'
                            ]),
                            '我们会同步软件中的配置到此脚本上，方便多个浏览器的管理。',
                            '窗口设置以及后台面板所有设置不会进行同步。'
                        ],
                        '如果不是，您可以忽略此脚本。'
                    ]).outerHTML
                },
                sync: {
                    defaultValue: false
                },
                connected: {
                    defaultValue: false
                },
                closeSync: {
                    defaultValue: false,
                    label: '关闭同步',
                    attrs: {
                        type: 'checkbox'
                    }
                }
            },
            onrender({ panel }) {
                // 同步面板不会被锁定
                panel.lockWrapper.remove();
                panel.configsContainer.classList.remove('lock');
                const update = () => {
                    if (this.cfg.sync) {
                        const tip = (0, easy_us_1.h)('div', { className: 'notes card' }, [`已成功同步软件中的配置.`]);
                        panel.body.replaceChildren((0, easy_us_1.h)('hr'), tip);
                    }
                    else if (this.cfg.connected) {
                        const tip = (0, easy_us_1.h)('div', { className: 'notes card' }, [`已成功连接到软件，但配置为空。`]);
                        panel.body.replaceChildren((0, easy_us_1.h)('hr'), tip);
                    }
                };
                update();
                this.offConfigChange(state.app.listenerIds.sync);
                this.offConfigChange(state.app.listenerIds.connected);
                this.offConfigChange(state.app.listenerIds.closeSync);
                state.app.listenerIds.sync = this.onConfigChange('sync', update);
                state.app.listenerIds.connected = this.onConfigChange('connected', update);
                state.app.listenerIds.closeSync = this.onConfigChange('closeSync', (closeSync) => {
                    if (closeSync) {
                        this.cfg.sync = false;
                        this.cfg.connected = false;
                        easy_us_1.$message.success({ content: '已关闭同步，刷新页面后生效' });
                    }
                });
            },
            onactive() {
                var _a;
                return __awaiter(this, void 0, void 0, function* () {
                    if (easy_us_1.$.isInTopWindow() && this.cfg.closeSync === false) {
                        this.cfg.sync = false;
                        try {
                            const res = yield (0, core_1.request)('http://localhost:15319/browser', {
                                type: 'GM_xmlhttpRequest',
                                method: 'get',
                                responseType: 'json'
                            });
                            this.cfg.connected = true;
                            if (res && Object.keys(res).length) {
                                // 排除几个特殊的设置
                                for (const key in res) {
                                    if (Object.prototype.hasOwnProperty.call(res, key)) {
                                        // 排除渲染脚本的设置
                                        if (render_1.RenderScript.namespace && key.startsWith(render_1.RenderScript.namespace)) {
                                            Reflect.deleteProperty(res, key);
                                        }
                                        // 排除后台脚本的设置
                                        for (const scriptKey in exports.BackgroundProject.scripts) {
                                            if (Object.prototype.hasOwnProperty.call(exports.BackgroundProject.scripts, scriptKey)) {
                                                const script = Reflect.get(exports.BackgroundProject.scripts, scriptKey);
                                                if (script.namespace && key.startsWith(script.namespace)) {
                                                    Reflect.deleteProperty(res, key);
                                                }
                                            }
                                        }
                                    }
                                }
                                // 排除那些不用同步的配置
                                for (const project of (0, __1.definedProjects)()) {
                                    for (const key in project.scripts) {
                                        if (Object.prototype.hasOwnProperty.call(project.scripts, key)) {
                                            const script = project.scripts[key];
                                            for (const ck in script.configs) {
                                                if (Object.prototype.hasOwnProperty.call(script.configs, ck)) {
                                                    if (((_a = script.configs[ck].extra) === null || _a === void 0 ? void 0 : _a.appConfigSync) === false) {
                                                        Reflect.deleteProperty(res, easy_us_1.$.namespaceKey(script.namespace, ck));
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                // 同步所有的配置
                                for (const key in res) {
                                    if (Object.prototype.hasOwnProperty.call(res, key)) {
                                        easy_us_1.$store.set(key, res[key]);
                                    }
                                }
                                // 锁定面板
                                for (const project of (0, __1.definedProjects)()) {
                                    // 排除后台脚本的锁定
                                    if (project.name === exports.BackgroundProject.name) {
                                        continue;
                                    }
                                    for (const key in project.scripts) {
                                        if (Object.prototype.hasOwnProperty.call(project.scripts, key)) {
                                            const script = project.scripts[key];
                                            const originalRender = script.onrender;
                                            // 重新定义渲染函数。在渲染后添加锁定面板的代码
                                            script.onrender = ({ panel, header }) => {
                                                var _a, _b;
                                                originalRender === null || originalRender === void 0 ? void 0 : originalRender({ panel, header });
                                                if (panel.configsContainer.children.length) {
                                                    panel.configsContainer.classList.add('lock');
                                                    panel.lockWrapper.style.width =
                                                        ((_a = panel.configsContainer.clientWidth) !== null && _a !== void 0 ? _a : panel.clientWidth) + 'px';
                                                    panel.lockWrapper.style.height =
                                                        ((_b = panel.configsContainer.clientHeight) !== null && _b !== void 0 ? _b : panel.clientHeight) + 'px';
                                                    panel.configsContainer.prepend(panel.lockWrapper);
                                                    panel.lockWrapper.title =
                                                        '🚫已同步OCS桌面版软件配置，如需修改请在桌面版软件的左侧栏设置-通用设置-OCS配置，中进行修改。或者前往脚本悬浮窗:后台-软件配置同步 关闭配置同步功能。';
                                                    panel.lockWrapper = easy_us_1.$ui.tooltip(panel.lockWrapper);
                                                }
                                            };
                                            // 重新执行渲染
                                            if (script.panel && script.header) {
                                                script.onrender({ panel: script.panel, header: script.header });
                                            }
                                        }
                                    }
                                }
                                this.cfg.sync = true;
                            }
                        }
                        catch (e) {
                            console.error(e);
                            this.cfg.sync = false;
                            this.cfg.connected = false;
                        }
                    }
                });
            }
        }),
        update: new easy_us_1.Script({
            name: '📥 更新模块',
            matches: [['所有页面', /.*/]],
            namespace: 'background.update',
            configs: {
                notes: {
                    defaultValue: '脚本自动更新模块，如果有新的版本会自动通知。'
                },
                autoNotify: {
                    defaultValue: true,
                    label: '开启更新通知',
                    attrs: { type: 'checkbox', title: '当有最新的版本时自动弹窗通知，默认开启' }
                },
                notToday: {
                    defaultValue: -1
                },
                ignoreVersions: {
                    defaultValue: []
                }
            },
            methods() {
                return {
                    getLastVersion: () => __awaiter(this, void 0, void 0, function* () {
                        return (yield (0, core_1.request)('https://cdn.ocsjs.com/ocs-version.json?t=' + Date.now(), {
                            method: 'get',
                            type: 'GM_xmlhttpRequest'
                        }));
                    })
                };
            },
            onrender({ panel }) {
                var _a;
                return __awaiter(this, void 0, void 0, function* () {
                    const version = yield this.methods.getLastVersion();
                    const infos = easy_us_1.$gm.getInfos();
                    if (!infos) {
                        return;
                    }
                    const changeLog = (0, easy_us_1.h)('button', { className: 'base-style-button-secondary' }, '📄查看更新日志');
                    changeLog.onclick = () => common_1.CommonProject.scripts.apps.methods.showChangelog();
                    const updatePage = ((_a = this.startConfig) === null || _a === void 0 ? void 0 : _a.updatePage) || '';
                    panel.body.replaceChildren((0, easy_us_1.h)('div', { className: 'card' }, [
                        (0, easy_us_1.h)('hr'),
                        (0, easy_us_1.h)('div', ['最新版本：' + version['last-version'] + ' - ', changeLog]),
                        (0, easy_us_1.h)('hr'),
                        (0, easy_us_1.h)('div', '当前版本：' + infos.script.version),
                        (0, easy_us_1.h)('div', '脚本管理器：' + infos.scriptHandler),
                        (0, easy_us_1.h)('div', ['脚本更新链接：', (0, easy_us_1.h)('a', { target: '_blank', href: updatePage }, [updatePage || '无'])])
                    ]));
                    console.log('versions', {
                        notToday: this.cfg.notToday,
                        ignoreVersions: this.cfg.ignoreVersions,
                        version: version
                    });
                });
            },
            oncomplete() {
                if (this.cfg.autoNotify && easy_us_1.$.isInTopWindow()) {
                    if (this.cfg.notToday === -1 || this.cfg.notToday !== new Date().getDate()) {
                        const infos = easy_us_1.$gm.getInfos();
                        if (infos) {
                            // 避免阻挡用户操作，这里等页面运行一段时间后再进行更新提示
                            setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                                var _a;
                                const version = yield this.methods.getLastVersion();
                                const last = version['last-version'];
                                if (
                                // 跳过主动忽略的版本
                                this.cfg.ignoreVersions.includes(last) === false &&
                                    // 版本比较
                                    (0, gt_1.default)(last, infos.script.version)) {
                                    const updatePage = ((_a = this.startConfig) === null || _a === void 0 ? void 0 : _a.updatePage) || '';
                                    const modal = easy_us_1.$modal.confirm({
                                        maskCloseable: false,
                                        width: 600,
                                        content: easy_us_1.$ui.notes([`检测到新版本发布 ${last} ：`, [...(version.notes || [])]]),
                                        footer: (0, easy_us_1.h)('div', [
                                            (0, easy_us_1.h)('button', { className: 'base-style-button-secondary', innerText: '跳过此版本' }, (btn) => {
                                                btn.onclick = () => {
                                                    this.cfg.ignoreVersions = [...this.cfg.ignoreVersions, last];
                                                    modal === null || modal === void 0 ? void 0 : modal.remove();
                                                };
                                            }),
                                            (0, easy_us_1.h)('button', { className: 'base-style-button-secondary', innerText: '今日不再提示' }, (btn) => {
                                                btn.onclick = () => {
                                                    this.cfg.notToday = new Date().getDate();
                                                    modal === null || modal === void 0 ? void 0 : modal.remove();
                                                };
                                            }),
                                            (0, easy_us_1.h)('button', { className: 'base-style-button', innerText: '前往更新' }, (btn) => {
                                                btn.onclick = () => {
                                                    if (updatePage) {
                                                        window.open(updatePage, '_blank');
                                                        modal === null || modal === void 0 ? void 0 : modal.remove();
                                                    }
                                                    else {
                                                        easy_us_1.$message.error({ content: '无法前往更新页面，更新链接为空' });
                                                    }
                                                };
                                            })
                                        ])
                                    });
                                }
                            }), 5 * 1000);
                        }
                    }
                }
            }
        }),
        dev: new easy_us_1.Script({
            name: '🛠️ 开发者调试',
            namespace: 'background.dev',
            matches: [['所有页面', /./]],
            configs: {
                notes: {
                    defaultValue: '开发人员调试用。<br>注入OCS_CONTEXT全局变量。用户可忽略此页面。'
                }
            },
            onrender({ panel }) {
                const injectBtn = (0, easy_us_1.h)('button', { className: 'base-style-button' }, '点击注入全局变量');
                injectBtn.addEventListener('click', () => {
                    easy_us_1.$gm.unsafeWindow.OCS_CONTEXT = self;
                });
                const showTabDataBtn = (0, easy_us_1.h)('button', { className: 'base-style-button' }, '显示Tab存储');
                easy_us_1.$gm.getTab((tab) => {
                    const els = [];
                    for (const key in tab) {
                        if (Object.prototype.hasOwnProperty.call(tab, key)) {
                            els.push((0, easy_us_1.h)('div', [(0, easy_us_1.h)('b', key + ' : '), (0, easy_us_1.h)('code', JSON.stringify(tab[key]))]));
                        }
                    }
                    showTabDataBtn.addEventListener('click', () => {
                        easy_us_1.$modal.simple({
                            content: (0, easy_us_1.h)('div', els),
                            width: window.document.documentElement.clientWidth / 2
                        });
                    });
                });
                panel.body.replaceChildren((0, easy_us_1.h)('div', { className: 'card' }, [injectBtn, showTabDataBtn]));
            }
        }),
        appLoginHelper: new easy_us_1.Script({
            name: '软件登录辅助',
            matches: [
                ['超星登录', 'passport2.chaoxing.com/login'],
                ['智慧树登录', 'passport.zhihuishu.com/login'],
                ['职教云登录', 'zjy2.icve.com.cn/portal/login.html'],
                ['智慧职教登录', 'sso.icve.com.cn/sso/auth']
            ],
            hideInPanel: true,
            oncomplete() {
                // 将面板移动至左侧顶部，防止挡住软件登录
                if (easy_us_1.$.isInTopWindow()) {
                    common_1.CommonProject.scripts.render.methods.moveToEdge();
                }
            }
        }),
        errorHandle: new easy_us_1.Script({
            name: '全局错误捕获',
            matches: [['', /.*/]],
            hideInPanel: true,
            onstart() {
                const projects = (0, __1.definedProjects)();
                for (const project of projects) {
                    for (const key in project.scripts) {
                        if (Object.prototype.hasOwnProperty.call(project.scripts, key)) {
                            const script = project.scripts[key];
                            script.on('scripterror', (err) => {
                                const msg = `[${project.name} - ${script.name}] : ${err}`;
                                console.error(msg);
                                exports.$console.error(msg);
                            });
                        }
                    }
                }
            }
        }),
        requestList: new easy_us_1.Script({
            name: '📄 请求记录',
            matches: [['', /.*/]],
            priority: 99,
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        '开发人员请求调试记录页面，小白勿入，最多只记录最近的100个请求数据',
                        '可打开F12控制台查看请求日志，或者下方的请求列表'
                    ]).outerHTML
                },
                enable: {
                    label: '开启请求记录',
                    attrs: { type: 'checkbox' },
                    defaultValue: false
                },
                methodFilter: {
                    label: '方法过滤',
                    tag: 'select',
                    attrs: { placeholder: '选择选项' },
                    options: [['none', '无'], ['GET'], ['POST'], ['OPTIONS'], ['HEAD']],
                    defaultValue: 'none'
                },
                typeFilter: {
                    label: '类型过滤',
                    tag: 'select',
                    attrs: { placeholder: '选择选项' },
                    options: [
                        ['none', '无'],
                        ['gmxhr', '油猴API请求（gmxhr）'],
                        ['fetch', '普通请求（fetch）']
                    ],
                    defaultValue: 'none'
                },
                searchValue: {
                    label: '内容搜索',
                    attrs: { placeholder: '搜索 URL/请求体/响应' },
                    defaultValue: ''
                },
                list: {
                    defaultValue: []
                }
            },
            methods() {
                const render = (list) => {
                    var _a, _b;
                    (_a = this.panel) === null || _a === void 0 ? void 0 : _a.body.replaceChildren();
                    (_b = this.panel) === null || _b === void 0 ? void 0 : _b.body.append((0, easy_us_1.h)('div', { className: 'card' }, [
                        (0, easy_us_1.h)('div', { style: { padding: '8px 0px', textAlign: 'end' } }, [
                            (0, easy_us_1.h)('button', {
                                className: 'base-style-button-secondary',
                                style: { marginRight: '12px' },
                                innerText: '🗑️清空记录'
                            }, (btn) => {
                                btn.onclick = () => {
                                    this.cfg.list = [];
                                    render(this.cfg.list);
                                };
                            }),
                            (0, easy_us_1.h)('button', { className: 'base-style-button', innerText: '🔍执行搜索' }, (btn) => {
                                btn.onclick = () => {
                                    if (this.cfg.methodFilter === 'none' &&
                                        this.cfg.typeFilter === 'none' &&
                                        this.cfg.searchValue === '') {
                                        render(this.cfg.list);
                                    }
                                    else {
                                        const list = this.cfg.list
                                            .filter((item) => {
                                            if (this.cfg.methodFilter !== 'none' &&
                                                item.method.toLowerCase() !== this.cfg.methodFilter.toLowerCase()) {
                                                return false;
                                            }
                                            return true;
                                        })
                                            .filter((item) => {
                                            if (this.cfg.typeFilter !== 'none' && item.type !== this.cfg.typeFilter) {
                                                return false;
                                            }
                                            return true;
                                        })
                                            .filter((item) => {
                                            var _a, _b;
                                            if ((this.cfg.searchValue && item.url.includes(this.cfg.searchValue)) ||
                                                ((_a = item.data) === null || _a === void 0 ? void 0 : _a.includes(this.cfg.searchValue)) ||
                                                ((_b = item.response) === null || _b === void 0 ? void 0 : _b.includes(this.cfg.searchValue))) {
                                                return true;
                                            }
                                            return false;
                                        });
                                        render(list);
                                    }
                                };
                            })
                        ]),
                        (0, easy_us_1.h)('div', { style: { backgroundColor: '#292929', overflow: 'auto', maxHeight: window.innerHeight / 2 + 'px' } }, [
                            ...(list.length === 0
                                ? [(0, easy_us_1.h)('div', { style: { color: 'white', textAlign: 'center' } }, '暂无数据')]
                                : []),
                            ...list.map((item) => 
                            // @ts-ignore
                            (0, easy_us_1.h)('div', {
                                title: Object.entries(item)
                                    .map(([key, val]) => key === 'time'
                                    ? `${key} : ${new Date(val).toLocaleString().replace(/\//g, '-')}`
                                    : `${key} : ${val}`)
                                    .join('\n'),
                                style: {
                                    maxWidth: '800px',
                                    padding: '4px 0px',
                                    margin: '4px 0px',
                                    // @ts-ignore
                                    textWrap: 'nowrap'
                                }
                            }, [
                                (0, easy_us_1.h)('div', [
                                    (0, easy_us_1.h)('span', { style: { marginRight: '8px' } }, new Date(item.time).toLocaleTimeString()),
                                    (0, easy_us_1.h)('span', {
                                        style: {
                                            backgroundColor: '#2196f3a3',
                                            color: '#ececec',
                                            marginRight: '8px',
                                            padding: '0px 2px'
                                        }
                                    }, item.method),
                                    (0, easy_us_1.h)('span', { style: { color: item.response ? '#4eb74e' : '#eb6262', marginRight: '8px' } }, '●'),
                                    (0, easy_us_1.h)('div', { style: { display: 'inline-block', color: '#ececec' } }, item.url ? (item.url.length > 100 ? item.url.slice(0, 100) + '...' : item.url) : '-')
                                ]),
                                (0, easy_us_1.h)('div', { style: { overflow: 'hidden', fontSize: '12px', color: '#8f8f8f' } }, item.data ? 'data: ' + item.data : ''),
                                (0, easy_us_1.h)('div', { style: { overflow: 'hidden', fontSize: '12px', color: '#8f8f8f' } }, item.response ? 'resp: ' + item.response : item.error ? 'err : ' + item.error : '')
                            ]))
                        ])
                    ]));
                };
                return {
                    render: render
                };
            },
            onrender() {
                this.methods.render(this.cfg.list);
            },
            onstart() {
                // 兼容其他环境
                if (easy_us_1.$gm.isInGMContext() === false) {
                    return;
                }
                /* global GM_xmlhttpRequest  RequestInfo RequestInit */
                /* eslint-disable no-global-assign */
                const gmRequest = GM_xmlhttpRequest;
                const originalFetch = fetch;
                const getId = () => Math.random().toString(16).slice(2);
                const addRecord = (item) => {
                    this.cfg.list = [item, ...this.cfg.list];
                    if (this.cfg.list.length > 100) {
                        this.cfg.list = this.cfg.list.slice(0, 100);
                    }
                };
                const setItem = (id, response, error) => {
                    const list = JSON.parse(JSON.stringify(this.cfg.list));
                    const index = list.findIndex((item) => item.id === id);
                    if (index !== -1) {
                        list[index].response = response;
                        list[index].error = error;
                    }
                    this.cfg.list = list;
                };
                // @ts-ignore
                GM_xmlhttpRequest = (details) => {
                    if (this.cfg.enable) {
                        const id = getId();
                        const data = {
                            id: id,
                            url: details.url,
                            method: details.method || 'unknown',
                            type: 'gmxhr',
                            data: details.data,
                            headers: details.headers,
                            response: '',
                            error: '',
                            time: Date.now()
                        };
                        addRecord(data);
                        const onload = details.onload;
                        const onerror = details.onerror;
                        details.onload = function (response) {
                            setItem(id, response.responseText, '');
                            data.response = details.responseType === 'json' ? response.response : response.responseText;
                            console.log('%c [请求成功]', 'color: green; font-weight: bold', data.url, data);
                            onload === null || onload === void 0 ? void 0 : onload.apply(this, [response]);
                        };
                        details.onerror = function (response) {
                            setItem(id, '', response.error);
                            data.error = response.error;
                            console.log('%c [请求失败]', 'color: red; font-weight: bold', data.url, data);
                            onerror === null || onerror === void 0 ? void 0 : onerror.apply(this, [response]);
                        };
                    }
                    return gmRequest.apply(this, [details]);
                };
                // @ts-ignore
                fetch = (input, init) => {
                    if (this.cfg.enable) {
                        const id = getId();
                        const data = {
                            id: id,
                            url: typeof input === 'string' ? input : input instanceof URL ? input.href : input.url,
                            method: (init === null || init === void 0 ? void 0 : init.method) || 'unknown',
                            type: 'fetch',
                            data: init === null || init === void 0 ? void 0 : init.body,
                            headers: init === null || init === void 0 ? void 0 : init.headers,
                            response: '',
                            error: '',
                            time: Date.now()
                        };
                        addRecord(data);
                        const res = originalFetch.apply(this, [input, init]);
                        res
                            .then((result) => {
                            return result.clone().text();
                        })
                            .then((result) => {
                            setItem(id, result, '');
                            data.response = result;
                            console.log('%c [请求成功]', 'color: green; font-weight: bold', data.url, data);
                        });
                        res.catch((err) => {
                            setItem(id, '', String(err));
                            data.error = String(err);
                            console.log('%c [请求失败]', 'color: red; font-weight: bold', data.url, data);
                        });
                        return res;
                    }
                    else {
                        return originalFetch.apply(this, [input, init]);
                    }
                };
            }
        })
    }
});
/** 日志对象，存储日志并显示在日志面板 */
exports.$console = new Proxy({}, {
    get(target, key) {
        return (...msg) => {
            var _a;
            let logs = exports.BackgroundProject.scripts.console.cfg.logs;
            if (logs.length > 50) {
                logs = logs.slice(-50);
            }
            const stack_str = Error().stack || '';
            // 简化堆栈信息
            const stacks = (_a = stack_str
                .replace('Error', '')
                .match(/at (.*) \(.+:\/\/.+:(.+):(.+)\)/g)) === null || _a === void 0 ? void 0 : _a.map((s) => {
                const match = s.match(/at (.*) \(.+:\/\/.+:(.+):(.+)\)/) || [];
                return [match[1], match[2], match[3]];
            });
            logs = logs.concat({
                type: key.toString(),
                content: msg.join(' '),
                time: Date.now(),
                stack: JSON.stringify([stack_str.split('\n')[0], ...(stacks || [])])
            });
            exports.BackgroundProject.scripts.console.cfg.logs = logs;
        };
    }
});
