import { SimplifyWorkResult, WorkUploadType } from '@ocsjs/core';
import { Project, Script, StoreListenerType } from 'easy-us';
import type { AnswerMatchMode, AnswererWrapper, SearchInformation } from '@ocsjs/core';
declare const state: {
    workResult: {
        /**
         * 题目位置同步处理器
         */
        questionPositionSyncHandler: {
            cx: (index: number) => void;
            'zhs-gxk': (index: number) => void;
            'zhs-xnk': (index: number) => void;
            icve: (index: number) => void;
            zjy: (index: number) => void;
            icourse: (index: number) => void;
        };
    };
    setting: {
        listenerIds: {
            aw: StoreListenerType;
        };
    };
};
/**
 * 题库缓存类型
 */
type QuestionCache = {
    title: string;
    answer: string;
    from: string;
    homepage: string;
};
export declare const CommonProject: Project<{
    guide: Script<import("easy-us").ScriptConfigs, import("easy-us").ScriptMethods>;
    settings: Script<{
        notes: {
            defaultValue: string;
        };
        notification: {
            label: string;
            attrs: {
                title: string;
            };
            tag: "select";
            defaultValue: "only-notify" | "notify-and-voice" | "all" | "no-notify";
            options: string[][];
        };
        enableQuestionCaches: {
            label: string;
            defaultValue: boolean;
            attrs: {
                type: string;
                title: string;
            };
        };
        answererWrappers: {
            separator: string;
            defaultValue: AnswererWrapper[];
        };
        /**
         * 禁用的题库
         */
        disabledAnswererWrapperNames: {
            defaultValue: string[];
        };
        answererWrappersButton: {
            label: string;
            defaultValue: string;
            attrs: {
                type: string;
            };
            onload(this: HTMLInputElement | HTMLSelectElement | HTMLTextAreaElement): void;
        };
        upload: {
            label: string;
            tag: "select";
            defaultValue: WorkUploadType;
            options: (string[] | [any, string, string])[];
            attrs: {
                title: string;
            };
        };
        stopSecondWhenFinish: {
            label: string;
            attrs: {
                type: string;
                min: number;
                step: number;
                max: number;
                title: string;
            };
            defaultValue: number;
        };
        thread: {
            label: string;
            attrs: {
                type: string;
                min: number;
                step: number;
                max: number;
                title: string;
            };
            defaultValue: number;
        };
        period: {
            label: string;
            attrs: {
                type: string;
                min: number;
                step: number;
                max: number;
                title: string;
            };
            defaultValue: number;
        };
        'work-when-no-job': {
            defaultValue: boolean;
            label: string;
            attrs: {
                type: string;
                title: string;
            };
        };
        'randomWork-choice': {
            defaultValue: boolean;
            label: string;
            attrs: {
                type: string;
                title: string;
            };
        };
        'randomWork-complete': {
            defaultValue: boolean;
            label: string;
            attrs: {
                type: string;
                title: string;
            };
        };
        'randomWork-completeTexts-textarea': {
            defaultValue: string;
            label: string;
            tag: "textarea";
            attrs: {
                title: string;
                style: {
                    minWidth: string;
                    minHeight: string;
                };
            };
            onload(this: HTMLInputElement | HTMLSelectElement | HTMLTextAreaElement, el: import("easy-us").ConfigElement<keyof import("easy-us/lib/elements/configs/interface").ConfigTagMap>): void;
        };
        answerSeparators: {
            label: string;
            attrs: {
                title: string;
            };
            defaultValue: string;
            onload(this: HTMLInputElement | HTMLSelectElement | HTMLTextAreaElement, el: import("easy-us").ConfigElement<keyof import("easy-us/lib/elements/configs/interface").ConfigTagMap>): void;
        };
        redundanceWordsText: {
            defaultValue: string;
            label: string;
            tag: "textarea";
            attrs: {
                title: string;
                style: {
                    minWidth: string;
                    minHeight: string;
                };
            };
            onload(this: HTMLInputElement | HTMLSelectElement | HTMLTextAreaElement, el: import("easy-us").ConfigElement<keyof import("easy-us/lib/elements/configs/interface").ConfigTagMap>): void;
        };
        answerMatchMode: {
            label: string;
            tag: "select";
            defaultValue: AnswerMatchMode;
            options: string[][];
        };
    }, {
        /**
         * 获取自动答题配置，包括题库配置
         */
        getWorkOptions: () => {
            notes: string;
            notification: "only-notify" | "notify-and-voice" | "all" | "no-notify";
            enableQuestionCaches: boolean;
            answererWrappers: AnswererWrapper[];
            disabledAnswererWrapperNames: string[];
            answererWrappersButton: string;
            upload: WorkUploadType;
            stopSecondWhenFinish: number;
            thread: number;
            period: number;
            'work-when-no-job': boolean;
            'randomWork-choice': boolean;
            'randomWork-complete': boolean;
            'randomWork-completeTexts-textarea': string;
            answerSeparators: string;
            redundanceWordsText: string;
            answerMatchMode: AnswerMatchMode;
        } & {
            notes?: string | undefined;
        };
        /**
         * 根据全局设置的配置，发起通知
         * @param content
         * @param opts
         */
        notificationBySetting: (content: string, opts?: {
            extraTitle?: string;
            /** 显示时间，单位为秒，默认为 30 秒， 0 则表示一直存在 */
            duration?: number;
            /** 通知点击时 */
            onclick?: () => void;
            /** 通知关闭时 */
            ondone?: () => void;
        }) => void;
    }>;
    workResults: Script<{
        notes: {
            defaultValue: string;
        };
        /**
         * 显示类型
         * list: 显示为题目列表
         * numbers: 显示为序号列表
         */
        type: {
            label: string;
            tag: "select";
            options: string[][];
            attrs: {
                title: string;
            };
            defaultValue: "numbers" | "questions";
        };
        totalQuestionCount: {
            defaultValue: number;
        };
        requestedCount: {
            defaultValue: number;
        };
        resolvedCount: {
            defaultValue: number;
        };
        currentResultIndex: {
            defaultValue: number;
        };
        questionPositionSyncHandlerType: {
            defaultValue: "cx" | "zhs-gxk" | "zhs-xnk" | "icve" | "zjy" | "icourse" | undefined;
        };
    }, {
        /**
         * 从搜索结果中计算状态，并更新
         */
        updateWorkStateByResults: (results: {
            requested: boolean;
            resolved: boolean;
        }[]) => void;
        /**
         * 更新状态
         */
        updateWorkState: (state: {
            totalQuestionCount: number;
            requestedCount: number;
            resolvedCount: number;
        }) => void;
        /**
         * 刷新状态
         */
        refreshState: () => void;
        /**
         * 清空搜索结果
         */
        clearResults: () => Promise<any>;
        getResults(): Promise<SimplifyWorkResult[]> | undefined;
        setResults(results: SimplifyWorkResult[]): Promise<any>;
        /**
         * 刷新搜索结果状态，清空搜索结果，置顶搜索结果面板
         */
        init(opts?: {
            questionPositionSyncHandlerType?: keyof typeof state.workResult.questionPositionSyncHandler;
        }): void;
        /**
         * 创建搜索结果面板
         * @param mount 挂载点
         */
        createWorkResultsPanel: (mount?: HTMLElement) => HTMLElement;
    }>;
    onlineSearch: Script<{
        notes: {
            defaultValue: string;
        };
        selectSearch: {
            label: string;
            defaultValue: boolean;
            attrs: {
                type: string;
                title: string;
            };
        };
        searchValue: {
            sync: true;
            label: string;
            tag: "textarea";
            attrs: {
                placeholder: string;
                style: {
                    minWidth: string;
                    minHeight: string;
                };
            };
            defaultValue: string;
        };
    }, import("easy-us").ScriptMethods>;
    /** 渲染脚本，窗口渲染主要脚本 */
    render: Script<any, {
        isPinned: (script: any) => Promise<boolean> | undefined;
        pin: (script: any) => Promise<void> | undefined;
        minimize: () => void | undefined;
        moveToEdge: () => void | undefined;
        normal: () => void;
    }>;
    hack: Script<import("easy-us").ScriptConfigs, import("easy-us").ScriptMethods>;
    disableDialog: Script<import("easy-us").ScriptConfigs, import("easy-us").ScriptMethods>;
    apps: Script<{
        notes: {
            defaultValue: string;
        };
        /**
         * 题库缓存
         */
        localQuestionCaches: {
            defaultValue: QuestionCache[];
            extra: {
                appConfigSync: boolean;
            };
        };
    }, {
        /**
         * 添加题库缓存
         */
        addQuestionCache: (...questionCacheItems: QuestionCache[]) => Promise<void>;
        addQuestionCacheFromWorkResult(swr: SimplifyWorkResult[]): void;
        /**
         * 将题库缓存作为题库并进行题目搜索
         * @param title 题目
         * @param whenSearchEmpty 当搜索结果为空，或者题库缓存功能被关闭时执行的函数
         */
        searchAnswerInCaches: (title: string, whenSearchEmpty: () => SearchInformation[] | Promise<SearchInformation[]>) => Promise<SearchInformation[]>;
        /**
         * 查看更新日志
         */
        showChangelog(): Promise<void>;
    }>;
}>;
export {};
