"use strict";
/** global Ext videojs getTeacherAjax jobs */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValidNumber = exports.fixedVideoProgress = exports.study = exports.CXProject = void 0;
const core_1 = require("@ocsjs/core");
const easy_us_1 = require("easy-us");
const common_1 = require("./common");
const configs_1 = require("../utils/configs");
const work_1 = require("../utils/work");
const md5_1 = __importDefault(require("md5"));
// @ts-ignore
const typr_js_1 = __importDefault(require("typr.js"));
const background_1 = require("./background");
const utils_1 = require("../utils");
try {
    /**
     *
     *  将繁体字映射载入内存。
     *  为什么不存 localStorage 和 GM_setValue
     *  localStorage: 存在被检测风险，谁都能访问
     *  GM_setValue: 文件太大影响I/O速度
     */
    // @ts-ignore
    top.typrMapping = top.typrMapping || undefined;
    // @ts-ignore 任务点
    top.jobs = top.jobs || [];
    // @ts-ignore 当前视频
    top.currentMedia = top.currentMedia || undefined;
    // 加 try 是因为跨域面板无法操作
}
catch (_a) { }
const state = {
    study: {
        videojs: Object.create({}),
        hacked: false,
        answererWrapperUnsetMessage: undefined,
        playbackRateWarningListenerId: 0
    }
};
exports.CXProject = easy_us_1.Project.create({
    name: '超星学习通',
    domains: [
        'chaoxing.com',
        'edu.cn',
        'org.cn',
        // 学银在线
        'xueyinonline.com',
        /** 其他域名 */
        'hnsyu.net',
        'qutjxjy.cn',
        'ynny.cn',
        'hnvist.cn',
        'fjlecb.cn',
        'gdhkmooc.com',
        'cugbonline.cn'
    ],
    scripts: {
        guide: new easy_us_1.Script({
            name: '💡 使用提示',
            matches: [
                ['首页', 'https://www.chaoxing.com'],
                ['旧版个人首页', 'chaoxing.com/space/index'],
                ['新版个人首页', 'chaoxing.com/base'],
                ['课程首页', 'chaoxing.com/mycourse'],
                ['新版课程首页', 'chaoxing.com/mooc2-ans/mycourse']
            ],
            namespace: 'cx.guide',
            configs: {
                notes: {
                    defaultValue: `请手动进入视频、作业、考试页面，脚本会自动运行。`
                }
            },
            oncomplete() {
                common_1.CommonProject.scripts.render.methods.pin(this);
            }
        }),
        study: new easy_us_1.Script({
            name: '🖥️ 课程学习',
            namespace: 'cx.new.study',
            matches: [
                ['任务点页面', '/knowledge/cards'],
                ['阅读任务点', '/readsvr/book/mooc']
                // 旧版浏览器好像不能识别二级 iframe ， 所以不能使用 'work/doHomeWorkNew' 以及其他二级 iframe 来触发路由
            ],
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        '自动答题前请在 “通用-全局设置” 中设置题库配置。',
                        ['任务点不是顺序执行，如果某一个任务没有动', '请查看是否有其他任务正在学习，耐心等待即可。'],
                        '闯关模式请注意题库如果没完成，需要自己完成才能解锁章节。',
                        '不要最小化浏览器，可能导致脚本暂停。',
                        '脚本详情运行日志请前往：后台-日志 查看'
                    ]).outerHTML
                },
                playbackRate: configs_1.playbackRate,
                volume: configs_1.volume,
                videoQuizStrategy: {
                    label: '视频内题目',
                    tag: 'select',
                    options: [
                        ['random', '随机答题'],
                        ['ignore', '忽略']
                    ],
                    attrs: {
                        title: '视频有时在学习过程中会弹出题目，这个好像并不计算在分数内，所以可以忽略，视频可以正常观看，这里提供几个方法处理题目'
                    },
                    defaultValue: 'random'
                },
                restudy: {
                    label: '复习模式',
                    attrs: { title: '已经完成的视频继续学习，并从当前的章节往下开始学习', type: 'checkbox' },
                    defaultValue: false
                },
                backToFirstWhenFinish: {
                    label: '完成全部后重新学习',
                    attrs: {
                        type: 'checkbox',
                        title: '当章节已经学习完成至最后一章时，跳转到第一个章节重新开始学习。'
                    },
                    defaultValue: false
                },
                autoNextPage: {
                    label: '自动下一章',
                    attrs: { type: 'checkbox' },
                    defaultValue: true
                },
                reloadVideoWhenError: {
                    label: '视频加载错误时自动刷新',
                    attrs: { type: 'checkbox' },
                    defaultValue: false
                },
                showTextareaWhenEdit: {
                    label: '编辑时显示自定义编辑框',
                    attrs: {
                        type: 'checkbox',
                        title: '超星默认禁止在编辑框中复制粘贴，开启此选项可以在文本框编辑时生成一个自定义编辑框进行编辑，脚本会将内容同步到编辑框中。'
                    },
                    defaultValue: true
                },
                notifyWhenHasFaceRecognition: {
                    label: '当有人脸验证时通知我',
                    attrs: {
                        type: 'checkbox'
                    },
                    defaultValue: true
                },
                /**
                 *
                 * 开启的任务点
                 *
                 * media : 音视频
                 * ppt : 文档和书籍翻阅
                 * test : 章节测试
                 * read : 阅读
                 * live : 直播课
                 *
                 */
                enableMedia: {
                    separator: '任务点开关',
                    label: '开启-视频/音频自动播放',
                    attrs: { type: 'checkbox', title: '开启：音频和视频的自动播放' },
                    defaultValue: true
                },
                enablePPT: {
                    label: '开启-PPT/书籍自动完成',
                    attrs: { type: 'checkbox', title: '开启：PPT/书籍自动翻阅' },
                    defaultValue: true
                },
                enableChapterTest: {
                    label: '开启-章节测试自动答题',
                    attrs: { type: 'checkbox', title: '开启：章节测试自动答题' },
                    defaultValue: true
                }
            },
            onrender({ panel }) {
                var _a;
                if (!((_a = common_1.CommonProject.scripts.settings.cfg.answererWrappers) === null || _a === void 0 ? void 0 : _a.length)) {
                    const setting = (0, easy_us_1.h)('button', { className: 'base-style-button-secondary' }, '通用-全局设置');
                    setting.onclick = () => common_1.CommonProject.scripts.render.methods.pin(common_1.CommonProject.scripts.settings);
                    if (state.study.answererWrapperUnsetMessage === undefined) {
                        state.study.answererWrapperUnsetMessage = easy_us_1.$message.warn({
                            content: (0, easy_us_1.h)('span', {}, ['检测到未设置题库配置，将无法自动答题，请切换到 ', setting, ' 页面进行配置。']),
                            duration: 0
                        });
                    }
                }
                // 高倍速警告
                this.offConfigChange(state.study.playbackRateWarningListenerId);
                state.study.playbackRateWarningListenerId =
                    this.onConfigChange('playbackRate', (playbackRate) => {
                        if (playbackRate > 3) {
                            easy_us_1.$modal.alert({
                                title: '⚠️高倍速警告',
                                content: easy_us_1.$ui.notes(['高倍速可能导致学习记录清空', '超星后台可以看到学习时长，请谨慎设置❗'])
                            });
                        }
                    }) || 0;
            },
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    /** iframe 跨域问题， 必须在 iframe 中执行 ， 所以脱离学习脚本运行。 */
                    if (/\/readsvr\/book\/mooc/.test(location.href)) {
                        background_1.$console.log('正在完成书籍/PPT...');
                        setTimeout(() => {
                            // @ts-ignore
                            // eslint-disable-next-line no-undef
                            readweb.goto(epage);
                        }, 5000);
                        return;
                    }
                    // 主要处理
                    if (/\/knowledge\/cards/.test(location.href)) {
                        const updateMediaState = () => {
                            // @ts-ignore
                            if (top.currentMedia) {
                                // @ts-ignore 倍速设置
                                top.currentMedia.playbackRate = parseFloat(this.cfg.playbackRate.toString());
                                // @ts-ignore 音量设置
                                top.currentMedia.volume = this.cfg.volume;
                            }
                        };
                        this.onConfigChange('playbackRate', updateMediaState);
                        this.onConfigChange('volume', updateMediaState);
                        yield study(Object.assign(Object.assign({}, this.cfg), { playbackRate: parseFloat(this.cfg.playbackRate.toString()), workOptions: common_1.CommonProject.scripts.settings.methods.getWorkOptions() }));
                    }
                });
            }
        }),
        work: new easy_us_1.Script({
            name: '✍️ 作业考试脚本',
            matches: [
                ['作业页面', '/mooc2/work/dowork'],
                ['考试整卷预览页面', '/mooc2/exam/preview']
            ],
            namespace: 'cx.new.work',
            configs: { notes: configs_1.workNotes },
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    const isExam = /\/exam\/preview/.test(location.href);
                    (0, work_1.commonWork)(this, {
                        workerProvider: (opts) => workOrExam(isExam ? 'exam' : 'work', opts)
                    });
                });
            }
        }),
        autoRead: new easy_us_1.Script({
            name: '🖥️ 自动阅读',
            matches: [
                ['阅读页面', '/ztnodedetailcontroller/visitnodedetail'],
                ['课程目录', /chaoxing.com\/course\/\d+\.html/],
                ['课程目录', /chaoxing.com\/mooc-ans\/course\/\d+\.html/]
            ],
            namespace: 'cx.new.auto-read',
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes(['阅读任务次日才会统计阅读时长']).outerHTML
                },
                restartAfterFinish: {
                    label: '无限阅读',
                    attrs: { type: 'checkbox', title: '阅读完成最后一章后从头第一章继续阅读' },
                    defaultValue: false
                }
            },
            oncomplete() {
                // 自动进入章节功能，如果不是阅读页面则自动进入
                if (location.href.includes('/ztnodedetailcontroller/visitnodedetail') === false) {
                    startAtFirst();
                    return;
                }
                let top = 0;
                const interval = setInterval(() => {
                    top += (document.documentElement.scrollHeight - window.innerHeight) / 60;
                    window.scrollTo({
                        behavior: 'smooth',
                        top: top
                    });
                }, 1000);
                setTimeout(() => {
                    clearInterval(interval);
                    // 下一页
                    const next = (0, easy_us_1.$el)('.nodeItem.r i');
                    if (next) {
                        next.click();
                    }
                    else {
                        if (this.cfg.restartAfterFinish) {
                            setTimeout(() => startAtFirst(), 3000);
                            easy_us_1.$message.info({ content: '即将重新从头开始阅读', duration: 10 });
                            background_1.$console.log('即将重新从头开始阅读');
                        }
                        else {
                            easy_us_1.$message.success({ content: '阅读任务已完成', duration: 0 });
                            background_1.$console.log('未检测到下一页');
                        }
                    }
                }, (60 + 3) * 1000);
                // 点击第一个章节
                function startAtFirst() {
                    const texts = (0, easy_us_1.$$el)('.course_section .chapterText');
                    if (texts.length) {
                        texts[0].click();
                    }
                }
            }
        }),
        /**
         * 有时候进入课程会默认在，任务页面，会出现任务为空，部分用户会以为没有章节任务，所以添加此脚本
         */
        pageRedirect: new easy_us_1.Script({
            name: '章节页面自动切换脚本',
            matches: [['课程任务页面', 'pageHeader=0']],
            hideInPanel: true,
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    if (top === window) {
                        const a = document.querySelector('a[title="章节"]');
                        if (a) {
                            yield core_1.$.sleep(1000);
                            // 跳转到最新版本的超星
                            a.click();
                            easy_us_1.$message.info({
                                content: '已经为您自动切换到章节列表页面，手动进入任意章节即可开始自动学习！'
                            });
                        }
                    }
                });
            }
        }),
        versionRedirect: new easy_us_1.Script({
            name: '版本切换脚本',
            matches: [
                ['', 'mooc2=0'],
                ['', 'mycourse/studentcourse'],
                ['', 'work/getAllWork'],
                ['', 'work/doHomeWorkNew'],
                ['', 'exam/test\\?'],
                ['', 'exam/test/reVersionTestStartNew.*examsystem.*'],
                ['', 'mooc-ans/mycourse/studentstudy']
            ],
            hideInPanel: true,
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    if (top === window) {
                        easy_us_1.$message.warn({
                            content: 'OCS网课助手不支持旧版超星, 即将切换到超星新版, 如有其他第三方插件请关闭, 可能有兼容问题导致频繁切换。'
                        });
                        // 跳转到最新版本的超星
                        yield core_1.$.sleep(2000);
                        const experience = document.querySelector('.experience');
                        if (experience) {
                            experience.click();
                        }
                        else {
                            const newUrl = new URL(window.location.href);
                            if (window.location.href.includes('mooc-ans/mycourse/studentstudy')) {
                                newUrl.pathname = '/mycourse/studentstudy';
                            }
                            const params = newUrl.searchParams;
                            params.set('mooc2', '1');
                            // 兼容考试切换
                            params.set('newMooc', 'true');
                            params.delete('examsystem');
                            window.location.replace(newUrl);
                        }
                    }
                });
            }
        }),
        examRedirect: new easy_us_1.Script({
            name: '考试整卷预览脚本',
            matches: [
                ['新版考试页面', 'exam-ans/exam/test/reVersionTestStartNew'],
                // 2023/9月 新增
                ['新版考试页面2', 'mooc-ans/exam/test/reVersionTestStartNew']
            ],
            hideInPanel: true,
            oncomplete() {
                easy_us_1.$message.info({ content: '即将跳转到整卷预览页面进行考试。' });
                setTimeout(() => easy_us_1.$gm.unsafeWindow.topreview(), 3000);
            }
        }),
        rateHack: new easy_us_1.Script({
            name: '屏蔽倍速限制',
            hideInPanel: true,
            matches: [['', '/ananas/modules/video/']],
            onstart() {
                rateHack();
            }
        }),
        copyHack: new easy_us_1.Script({
            name: '屏蔽复制粘贴限制',
            hideInPanel: true,
            matches: [['所有页面', /.*/]],
            methods() {
                return {
                    /** 解除输入框无法复制粘贴 */
                    hackEditorPaste() {
                        var _a, _b;
                        try {
                            const instants = ((_b = (_a = easy_us_1.$gm.unsafeWindow) === null || _a === void 0 ? void 0 : _a.UE) === null || _b === void 0 ? void 0 : _b.instants) || [];
                            for (const key in instants) {
                                const ue = instants[key];
                                /**
                                 * 新建一个文本框给用户编辑，然后同步到超星编辑器，防止http下浏览器无法读取剪贴板
                                 */
                                // eslint-disable-next-line no-proto
                                if (ue === null || ue === void 0 ? void 0 : ue.textarea) {
                                    ue.body.addEventListener('click', () => __awaiter(this, void 0, void 0, function* () {
                                        // http 下无法读取剪贴板，通过弹窗让用户输入然后同步到编辑器
                                        if (exports.CXProject.scripts.study.cfg.showTextareaWhenEdit) {
                                            const defaultText = (0, easy_us_1.h)('span', { innerHTML: ue.textarea.value }).textContent;
                                            easy_us_1.$modal.prompt({
                                                content: '请在此文本框进行编辑，防止超星无法复制粘贴。(如需关闭请前往设置: 课程学习-编辑时显示自定义编辑框)',
                                                width: 800,
                                                inputDefaultValue: defaultText || '',
                                                modalInputType: 'textarea',
                                                onConfirm: (val = '') => {
                                                    ue.setContent(val
                                                        .split('\n')
                                                        .map((line) => `<p>${line}</p>`)
                                                        .join(''));
                                                }
                                            });
                                        }
                                    }));
                                    if (easy_us_1.$gm.unsafeWindow.editorPaste) {
                                        ue.removeListener('beforepaste', easy_us_1.$gm.unsafeWindow.editorPaste);
                                    }
                                    if (easy_us_1.$gm.unsafeWindow.myEditor_paste) {
                                        ue.removeListener('beforepaste', easy_us_1.$gm.unsafeWindow.myEditor_paste);
                                    }
                                }
                            }
                        }
                        catch (_c) { }
                    }
                };
            },
            oncomplete() {
                const hackInterval = setInterval(() => {
                    if (typeof easy_us_1.$gm.unsafeWindow.UE !== 'undefined') {
                        clearInterval(hackInterval);
                        this.methods.hackEditorPaste();
                        console.log('已解除输入框无法复制粘贴限制');
                    }
                }, 500);
            }
        }),
        studyDispatcher: new easy_us_1.Script({
            name: '课程学习调度器',
            matches: [['课程学习页面', '/mycourse/studentstudy']],
            namespace: 'cx.new.study-dispatcher',
            hideInPanel: true,
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, easy_us_1.$menu)('🖥️', { scriptPanelLink: exports.CXProject.scripts.study });
                    (0, easy_us_1.$menu)('⚙️', { scriptPanelLink: common_1.CommonProject.scripts.settings });
                    (0, easy_us_1.$menu)('🌏', { scriptPanelLink: common_1.CommonProject.scripts.workResults });
                    (0, easy_us_1.$menu)('📄', { scriptPanelLink: background_1.BackgroundProject.scripts.console });
                    (0, easy_us_1.$menu)('📥', { scriptPanelLink: background_1.BackgroundProject.scripts.update });
                    // 开始任务切换
                    const restudy = exports.CXProject.scripts.study.cfg.restudy;
                    common_1.CommonProject.scripts.render.methods.pin(exports.CXProject.scripts.study);
                    if (!restudy) {
                        // 如果不是复习模式，则寻找需要运行的任务
                        const params = new URLSearchParams(window.location.href);
                        const mooc = params.get('mooc2');
                        /** 切换新版 */
                        if (mooc === null) {
                            params.set('mooc2', '1');
                            window.location.replace(decodeURIComponent(params.toString()));
                            return;
                        }
                        let chapters = yield CXAnalyses.waitForChapterInfos();
                        // 过滤掉已完成的章节
                        chapters = chapters.filter((chapter) => chapter.unFinishCount !== 0);
                        if (chapters.length === 0) {
                            easy_us_1.$message.warn({ content: '页面任务点数量为空! 请刷新重试!' });
                        }
                        else {
                            const params = new URLSearchParams(window.location.href);
                            const courseId = params.get('courseId');
                            const classId = params.get('clazzid');
                            setTimeout(() => {
                                //  进入需要进行的章节，并且当前章节未被选中
                                if ((0, easy_us_1.$$el)(`.posCatalog_active[id="cur${chapters[0].chapterId}"]`).length === 0) {
                                    easy_us_1.$gm.unsafeWindow.getTeacherAjax(courseId, classId, chapters[0].chapterId);
                                }
                            }, 1000);
                        }
                    }
                });
            }
        }),
        cxSecretFontRecognize: new easy_us_1.Script({
            name: '繁体字识别',
            hideInPanel: true,
            matches: [
                ['题目页面', 'work/doHomeWorkNew'],
                ['考试整卷预览', '/mooc2/exam/preview'],
                ['作业', '/mooc2/work/dowork']
            ],
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    yield mappingRecognize();
                });
            }
        })
    }
});
function workOrExam(type = 'work', { answererWrappers, period, thread, redundanceWordsText, answerSeparators, answerMatchMode }) {
    easy_us_1.$message.info(`开始${type === 'work' ? '作业' : '考试'}`);
    common_1.CommonProject.scripts.workResults.methods.init({
        questionPositionSyncHandlerType: 'cx'
    });
    // 处理作业和考试题目的方法
    const workOrExamQuestionTitleTransform = (titles) => {
        const optimizationTitle = titles
            .map((titleElement) => {
            if (titleElement) {
                const titleCloneEl = titleElement.cloneNode(true);
                const childNodes = titleCloneEl.childNodes;
                // 删除序号
                childNodes[0].remove();
                // 删除题型
                childNodes[0].remove();
                // 显示图片链接在题目中
                return (0, work_1.optimizationElementWithImage)(titleCloneEl).innerText;
            }
            return '';
        })
            .join(',');
        return (0, work_1.removeRedundantWords)(core_1.StringUtils.of(optimizationTitle).nowrap().nospace().toString().trim(), redundanceWordsText.split('\n'));
    };
    /** 新建答题器 */
    const worker = new core_1.OCSWorker({
        root: '.questionLi',
        elements: {
            title: [
                /** 题目标题 */
                (root) => (0, easy_us_1.$el)('h3', root)
                // /** 连线题第一组 */
                // (root) => $el('.line_wid_half.fl', root),
                // /** 连线题第二组 */
                // (root) => $el('.line_wid_half.fr', root)
            ],
            options: '.answerBg .answer_p, .textDIV, .eidtDiv',
            type: type === 'exam' ? 'input[name^="type"]' : 'input[id^="answertype"]',
            lineAnswerInput: '.line_answer input[name^=answer]',
            lineSelectBox: '.line_answer_ct .selectBox ',
            /** 阅读理解 */
            reading: '.reading_answer',
            /** 完形填空 */
            filling: '.filling_answer'
        },
        thread: thread !== null && thread !== void 0 ? thread : 1,
        answerSeparators: answerSeparators.split(',').map((s) => s.trim()),
        answerMatchMode: answerMatchMode,
        /** 默认搜题方法构造器 */
        answerer: (elements, ctx) => {
            if (elements.title) {
                // 处理作业和考试题目
                const title = workOrExamQuestionTitleTransform(elements.title);
                if (title) {
                    const typeInput = elements.type[0];
                    return common_1.CommonProject.scripts.apps.methods.searchAnswerInCaches(title, () => __awaiter(this, void 0, void 0, function* () {
                        yield core_1.$.sleep((period !== null && period !== void 0 ? period : 3) * 1000);
                        return (0, core_1.defaultAnswerWrapperHandler)(answererWrappers, {
                            type: (typeInput ? getQuestionType(parseInt(typeInput.value)) : undefined) || 'unknown',
                            title,
                            options: ctx.elements.options.map((o) => o.innerText).join('\n')
                        });
                    }));
                }
                else {
                    throw new Error('题目为空，请查看题目是否为空，或者忽略此题');
                }
            }
            else {
                throw new Error('题目为空，请查看题目是否为空，或者忽略此题');
            }
        },
        work: (ctx) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const { elements, searchInfos } = ctx;
            const typeInput = elements.type[0];
            const type = getQuestionType(parseInt(typeInput.value));
            if (type && (type === 'completion' || type === 'multiple' || type === 'judgement' || type === 'single')) {
                const resolver = (0, core_1.defaultQuestionResolve)(ctx)[type];
                return yield resolver(searchInfos, elements.options.map((option) => (0, work_1.optimizationElementWithImage)(option)), (type, answer, option) => __awaiter(this, void 0, void 0, function* () {
                    var _b, _c, _d;
                    // 如果存在已经选择的选项
                    if (type === 'judgement' || type === 'single' || type === 'multiple') {
                        if ((option === null || option === void 0 ? void 0 : option.parentElement) && (0, easy_us_1.$$el)('[class*="check_answer"]', option.parentElement).length === 0) {
                            option.click();
                            yield core_1.$.sleep(500);
                        }
                    }
                    else if (type === 'completion' && answer.trim()) {
                        const text = option === null || option === void 0 ? void 0 : option.querySelector('textarea');
                        const textareaFrame = option === null || option === void 0 ? void 0 : option.querySelector('iframe');
                        if (text) {
                            text.value = answer;
                        }
                        if (textareaFrame === null || textareaFrame === void 0 ? void 0 : textareaFrame.contentDocument) {
                            textareaFrame.contentDocument.body.innerHTML = answer;
                        }
                        if ((_b = option === null || option === void 0 ? void 0 : option.parentElement) === null || _b === void 0 ? void 0 : _b.parentElement) {
                            /** 如果存在保存按钮则点击 */
                            (_d = (0, easy_us_1.$el)('[onclick*=saveQuestion]', (_c = option === null || option === void 0 ? void 0 : option.parentElement) === null || _c === void 0 ? void 0 : _c.parentElement)) === null || _d === void 0 ? void 0 : _d.click();
                            yield core_1.$.sleep(500);
                        }
                    }
                }));
            }
            // 连线题自定义处理
            else if (type && type === 'line') {
                for (const answers of searchInfos.map((info) => info.results.map((res) => res.answer))) {
                    let ans = answers;
                    if (ans.length === 1) {
                        ans = (0, core_1.splitAnswer)(ans[0]);
                    }
                    if (ans.filter(Boolean).length !== 0 && elements.lineAnswerInput) {
                        //  选择答案
                        for (let index = 0; index < elements.lineSelectBox.length; index++) {
                            const box = elements.lineSelectBox[index];
                            if (ans[index]) {
                                (_a = (0, easy_us_1.$el)(`li[data=${ans[index]}] a`, box)) === null || _a === void 0 ? void 0 : _a.click();
                                yield core_1.$.sleep(200);
                            }
                        }
                        return { finish: true };
                    }
                }
                return { finish: false };
            }
            // 完形填空
            else if (type && type === 'fill') {
                return readerAndFillHandle(searchInfos, elements.filling);
            }
            // 阅读理解
            else if (type && type === 'reader') {
                return readerAndFillHandle(searchInfos, elements.reading);
            }
            return { finish: false };
        }),
        /** 完成答题后 */
        onResultsUpdate(current, _, res) {
            var _a;
            common_1.CommonProject.scripts.workResults.methods.setResults((0, work_1.simplifyWorkResult)(res, workOrExamQuestionTitleTransform));
            common_1.CommonProject.scripts.workResults.methods.updateWorkStateByResults(res);
            if ((_a = current.result) === null || _a === void 0 ? void 0 : _a.finish) {
                common_1.CommonProject.scripts.apps.methods.addQuestionCacheFromWorkResult((0, work_1.simplifyWorkResult)([current], workOrExamQuestionTitleTransform));
            }
        }
    });
    worker
        .doWork()
        .then(() => {
        easy_us_1.$message.info({ content: '作业/考试完成，请自行检查后保存或提交。', duration: 0 });
        worker.emit('done');
    })
        .catch((err) => {
        console.error(err);
        easy_us_1.$message.error('答题程序发生错误 : ' + err.message);
    });
    return worker;
}
/**
 * 繁体字识别-字典匹配
 * @see 参考 https://bbs.tampermonkey.net.cn/thread-2303-1-1.html
 */
function mappingRecognize(doc = document) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        let typrMapping = Object.create({});
        try {
            // @ts-ignore
            top.typrMapping = top.typrMapping || (yield loadTyprMapping());
            // @ts-ignore
            typrMapping = top.typrMapping;
        }
        catch (_c) {
            // 超星考试可能嵌套其他平台中，所以会存在跨域，这里需要处理一下跨域情况，如果是跨域直接在当前页面加载字库
            typrMapping = yield loadTyprMapping();
        }
        /** 判断是否有繁体字 */
        const fontFaceEl = Array.from(doc.head.querySelectorAll('style')).find((style) => { var _a; return (_a = style.textContent) === null || _a === void 0 ? void 0 : _a.includes('font-cxsecret'); });
        const base64ToUint8Array = (base64) => {
            const data = window.atob(base64);
            const buffer = new Uint8Array(data.length);
            for (let i = 0; i < data.length; ++i) {
                buffer[i] = data.charCodeAt(i);
            }
            return buffer;
        };
        const fontMap = typrMapping;
        if (fontFaceEl && Object.keys(fontMap).length > 0) {
            // 解析font-cxsecret字体
            const font = (_b = (_a = fontFaceEl.textContent) === null || _a === void 0 ? void 0 : _a.match(/base64,([\w\W]+?)'/)) === null || _b === void 0 ? void 0 : _b[1];
            if (font) {
                background_1.$console.log('正在识别繁体字');
                const code = typr_js_1.default.parse(base64ToUint8Array(font));
                // 匹配解密字体
                const match = {};
                for (let i = 19968; i < 40870; i++) {
                    // 中文[19968, 40869]
                    const Glyph = typr_js_1.default.U.codeToGlyph(code, i);
                    if (!Glyph)
                        continue;
                    const path = typr_js_1.default.U.glyphToPath(code, Glyph);
                    const hex = (0, md5_1.default)(JSON.stringify(path)).slice(24); // 8位即可区分
                    match[i.toString()] = fontMap[hex];
                }
                const fonts = CXAnalyses.getSecretFont(doc);
                // 替换加密字体
                fonts.forEach((el, index) => {
                    let html = el.innerHTML;
                    for (const key in match) {
                        const word = String.fromCharCode(parseInt(key));
                        const value = String.fromCharCode(match[key]);
                        while (html.indexOf(word) !== -1) {
                            html = html.replace(word, value);
                        }
                    }
                    el.innerHTML = html;
                    el.classList.remove('font-cxsecret'); // 移除字体加密
                });
                background_1.$console.log('识别繁体字完成。');
            }
            else {
                background_1.$console.log('未检测到繁体字。');
            }
        }
    });
}
function loadTyprMapping() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            background_1.$console.log('正在加载繁体字库。');
            return yield (0, core_1.request)('https://cdn.ocsjs.com/resources/font/table.json', {
                type: 'GM_xmlhttpRequest',
                method: 'get',
                responseType: 'json'
            });
        }
        catch (err) {
            background_1.$console.error('载繁体字库加载失败，请刷新页面重试：', String(err));
        }
    });
}
/**
 * cx分析工具
 */
const CXAnalyses = {
    /** 是否处于闯关模式或者解锁模式 */
    isInSpecialMode() {
        return Array.from((top === null || top === void 0 ? void 0 : top.document.querySelectorAll('.catalog_points_sa,.catalog_points_er')) || []).length !== 0;
    },
    /** 是否为闯关模式，并且当前章节卡在最后一个待完成的任务点 */
    isStuckInBreakingMode() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isInSpecialMode()) {
                const chapter = top === null || top === void 0 ? void 0 : top.document.querySelector('.posCatalog_active');
                if (chapter) {
                    const id = chapter.getAttribute('id');
                    if (id) {
                        // 超星好像会重绘组件，导致无法绑定属性到元素中，所以这里使用页面全局变量存储
                        const counter = (yield easy_us_1.$store.getTab('chapter_counter')) || Object.create({});
                        let count = Reflect.get(counter, id);
                        count = count ? count + 1 : 1;
                        Reflect.set(counter, id, count);
                        let res = false;
                        if (count >= 3) {
                            Reflect.set(counter, id, 1);
                            res = true;
                        }
                        yield easy_us_1.$store.setTab('chapter_counter', counter);
                        return res;
                    }
                }
            }
            return false;
        });
    },
    /** 是否处于最后一小节 */
    isInFinalTab() {
        // 上方小节任务栏
        const tabs = Array.from((top === null || top === void 0 ? void 0 : top.document.querySelectorAll('.prev_ul li')) || []);
        return tabs.length && tabs[tabs.length - 1].classList.contains('active');
    },
    /** 是否处于最后一个章节 */
    isInFinalChapter() {
        var _a;
        return (_a = Array.from((top === null || top === void 0 ? void 0 : top.document.querySelectorAll('.posCatalog_select')) || [])
            .pop()) === null || _a === void 0 ? void 0 : _a.classList.contains('posCatalog_active');
    },
    /** 是否完成全部章节 */
    isFinishedAllChapters() {
        return this.getChapterInfos().every((chapter) => chapter.unFinishCount === 0);
    },
    /** 获取所有章节信息 */
    getChapterInfos() {
        return Array.from((top === null || top === void 0 ? void 0 : top.document.querySelectorAll('[onclick^="getTeacherAjax"]')) || []).map((el) => {
            var _a, _b, _c;
            return ({
                chapterId: (_b = (_a = el.getAttribute('onclick')) === null || _a === void 0 ? void 0 : _a.match(/\('(.*)','(.*)','(.*)'\)/)) === null || _b === void 0 ? void 0 : _b[3],
                // @ts-ignore
                unFinishCount: parseInt(((_c = el.parentElement.querySelector('.jobUnfinishCount')) === null || _c === void 0 ? void 0 : _c.value) || '0')
            });
        });
    },
    /**
     * 等待并获取章节信息，直到获取到章节信息为止
     * - 可设置超时时间（单位秒），默认10秒
     * - 超时后返回空数组
     */
    waitForChapterInfos(timeout = 10) {
        return new Promise((resolve, reject) => {
            const interval = setInterval(() => {
                const res = this.getChapterInfos();
                if (res.length > 0) {
                    clearInterval(interval);
                    clearInterval(to);
                    resolve(res);
                }
            }, 1000);
            const to = setTimeout(() => {
                clearInterval(interval);
                resolve([]);
            }, timeout * 1000);
        });
    },
    /** 检测页面是否使用字体加密 */
    getSecretFont(doc = document) {
        return Array.from(doc.querySelectorAll('.font-cxsecret')).map((font) => {
            // 这里吧选项按钮和文字分离，如果不分离的话 .font-cxsecret 元素下面还包含选项按钮时，替换时会吧按钮也删除掉导致选项按钮不可用
            const after = font.querySelector('.after');
            return after === null ? font : after;
        });
    }
};
/**
 * 屏蔽倍速限制
 */
function rateHack() {
    state.study.hacked = false;
    let dragCount = 0;
    try {
        hack();
        window.document.addEventListener('readystatechange', hack);
        window.addEventListener('load', hack);
    }
    catch (e) {
        console.error(e);
    }
    function hack() {
        const videojs = easy_us_1.$gm.unsafeWindow.videojs;
        const Ext = easy_us_1.$gm.unsafeWindow.Ext;
        if (typeof videojs !== 'undefined' && typeof Ext !== 'undefined') {
            if (state.study.hacked) {
                return;
            }
            state.study.hacked = true;
            const _origin = videojs.getPlugin('seekBarControl');
            const plugin = videojs.extend(videojs.getPlugin('plugin'), {
                constructor: function (videoExt, data) {
                    const _sendLog = data.sendLog;
                    data.sendLog = (...args) => {
                        var _a;
                        if (args[1] === 'drag') {
                            dragCount++;
                            // 开始播放的时候偶尔会卡顿，导致一直触发 drag 事件（超星的BUG）
                            // 这里如果卡顿太多，尝试暂停视频，然后等待视频自动开始。
                            if (dragCount > 100) {
                                dragCount = 0;
                                (_a = (0, easy_us_1.$el)('video')) === null || _a === void 0 ? void 0 : _a.pause();
                            }
                        }
                        else {
                            _sendLog.apply(data, args);
                        }
                    };
                    _origin.apply(_origin.prototype, [videoExt, data]);
                }
            });
            videojs.registerPlugin('seekBarControl', plugin);
            // 重写超星视频插件
            Ext.define('ans.VideoJs', {
                override: 'ans.VideoJs',
                constructor: function (data) {
                    this.addEvents(['seekstart']);
                    this.mixins.observable.constructor.call(this, data);
                    const vjs = videojs(data.videojs, this.params2VideoOpt(data.params), function () { });
                    Ext.fly(data.videojs).on('contextmenu', function (f) {
                        f.preventDefault();
                    });
                    Ext.fly(data.videojs).on('keydown', function (f) {
                        if (f.keyCode === 32 || f.keyCode === 37 || f.keyCode === 39 || f.keyCode === 107) {
                            f.preventDefault();
                        }
                    });
                    // 保存清晰度设置
                    if (vjs.videoJsResolutionSwitcher) {
                        vjs.on('resolutionchange', function () {
                            const cr = vjs.currentResolution();
                            const re = cr.sources ? cr.sources[0].res : false;
                            Ext.setCookie('resolution', re);
                        });
                    }
                    // 保存公网设置
                    if (vjs.videoJsPlayLine) {
                        vjs.on('playlinechange', function () {
                            const cp = vjs.currentPlayline();
                            Ext.setCookie('net', cp.net);
                        });
                    }
                    // 下面连着一个倍速限制方法，这里直接不写，实现可以倍速
                }
            });
        }
    }
}
/**
 * cx 任务学习
 */
function study(opts) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        yield core_1.$.sleep(3000);
        const searchedJobs = [];
        let searching = true;
        let attachmentCount = ((_a = easy_us_1.$gm.unsafeWindow.attachments) === null || _a === void 0 ? void 0 : _a.length) || 0;
        /** 考虑到网速级慢的同学，所以10秒后如果还没有任务点才停止 */
        setTimeout(() => {
            searching = false;
        }, 10 * 1000);
        /**
         * 递归运行任务点，一旦有新的任务点被检测到直接开始
         * 如果10秒内既没有任务点，也暂停了搜索，则当前则没有任务点
         */
        const runJobs = () => __awaiter(this, void 0, void 0, function* () {
            const job = searchJob(opts, searchedJobs);
            // 如果存在任务点
            if (job && job.func) {
                try {
                    yield job.func();
                }
                catch (e) {
                    background_1.$console.error('未知错误', e);
                }
                yield core_1.$.sleep(1000);
                yield runJobs();
            }
            // 每次 search 一次，就减少一次文件数量
            // 如果不加这个判断，三个任务中，中间的任务不是任务点，则会导致下面的任务全部不执行。
            else if (attachmentCount > 0) {
                attachmentCount--;
                yield core_1.$.sleep(1000);
                yield runJobs();
            }
            // 或者正在搜索
            else if (searching) {
                yield core_1.$.sleep(1000);
                yield runJobs();
            }
        });
        yield runJobs();
        // @ts-ignore
        top._preChapterId = '';
        // 下一章
        const next = () => __awaiter(this, void 0, void 0, function* () {
            var _b;
            const curCourseId = (0, easy_us_1.$el)('#curCourseId', top === null || top === void 0 ? void 0 : top.document);
            const curChapterId = (0, easy_us_1.$el)('#curChapterId', top === null || top === void 0 ? void 0 : top.document);
            const curClazzId = (0, easy_us_1.$el)('#curClazzId', top === null || top === void 0 ? void 0 : top.document);
            const count = (0, easy_us_1.$$el)('#prev_tab .prev_ul li', top === null || top === void 0 ? void 0 : top.document);
            if (CXAnalyses.isInFinalTab()) {
                if (yield CXAnalyses.isStuckInBreakingMode()) {
                    return easy_us_1.$modal.alert({
                        content: '检测到此章节重复进入, 为了避免无限重复, 请自行手动完成后手动点击下一章, 或者刷新重试。'
                    });
                }
            }
            if (CXAnalyses.isInFinalChapter()) {
                let content = '';
                if (opts.backToFirstWhenFinish) {
                    content = '已经抵达最后一个章节，10秒后返回第一个章节重新开始。';
                    setTimeout(() => {
                        var _a;
                        (_a = top === null || top === void 0 ? void 0 : top.document.querySelector('.posCatalog_name')) === null || _a === void 0 ? void 0 : _a.click();
                    }, 10 * 1000);
                    easy_us_1.$message.info({ content, duration: 30 });
                }
                else {
                    if (CXAnalyses.isFinishedAllChapters()) {
                        content = '全部任务点已完成！';
                    }
                    else {
                        content = '已经抵达最后一个章节！但仍然有任务点未完成，请手动切换至未完成的章节。';
                    }
                    easy_us_1.$modal.alert({ content: content });
                }
                common_1.CommonProject.scripts.settings.methods.notificationBySetting(content, {
                    duration: 0,
                    extraTitle: '超星学习通学习脚本'
                });
            }
            else {
                if (curChapterId && curCourseId && curClazzId) {
                    // @ts-ignore
                    top._preChapterId = curChapterId.value;
                    /**
                     * count, chapterId, courseId, clazzid, knowledgestr, checkType
                     * checkType 就是询问当前章节还有任务点未完成，是否完成，这里直接不传，默认下一章
                     */
                    // @ts-ignore
                    (_b = easy_us_1.$gm.unsafeWindow.top) === null || _b === void 0 ? void 0 : _b.PCount.next(count.length.toString(), curChapterId.value, curCourseId.value, curClazzId.value, '');
                }
                else {
                    background_1.$console.warn('参数错误，无法跳转下一章，请尝试手动切换。');
                }
            }
        });
        if (exports.CXProject.scripts.study.cfg.autoNextPage) {
            const msg = '页面任务点已完成，即将切换下一章。';
            easy_us_1.$message.success({ content: msg });
            background_1.$console.info(msg);
            yield core_1.$.sleep(5000);
            next();
        }
        else {
            const msg = '页面任务点已完成，自动下一章已关闭，请手动切换。';
            easy_us_1.$message.warn({ content: msg });
            background_1.$console.warn(msg);
        }
    });
}
exports.study = study;
function searchIFrame(root) {
    var _a, _b;
    let list = Array.from(root.querySelectorAll('iframe'));
    const result = [];
    while (list.length) {
        const frame = list.shift();
        try {
            if (frame && ((_a = frame === null || frame === void 0 ? void 0 : frame.contentWindow) === null || _a === void 0 ? void 0 : _a.document)) {
                result.push(frame);
                const frames = (_b = frame === null || frame === void 0 ? void 0 : frame.contentWindow) === null || _b === void 0 ? void 0 : _b.document.querySelectorAll('iframe');
                list = list.concat(Array.from(frames || []));
            }
        }
        catch (e) {
            // @ts-ignore
            console.log(e.message);
        }
    }
    return result;
}
/**
 * 搜索任务点
 */
function searchJob(opts, searchedJobs) {
    const knowCardWin = easy_us_1.$gm.unsafeWindow;
    const searchJobElement = (root) => {
        return (0, core_1.domSearch)({
            media: 'video,audio',
            chapterTest: '.TiMu',
            read: '#img.imglook'
        }, root.contentWindow.document);
    };
    const search = (root) => {
        const win = root.contentWindow;
        const { media, read, chapterTest } = searchJobElement(root);
        if (win && (media || read || chapterTest)) {
            const doc = win.document;
            const attachment = 
            // @ts-ignore
            knowCardWin.attachments[getValidNumber(win._jobindex, win.parent._jobindex)];
            // 任务点去重
            if (attachment && searchedJobs.find((job) => job.mid === attachment.property.mid) === undefined) {
                const { name, title, bookname, author } = attachment.property;
                const jobName = name || title || (bookname ? bookname + author : undefined) || '未知任务';
                let func;
                if (media) {
                    if (!exports.CXProject.scripts.study.cfg.enableMedia) {
                        const msg = `音视频自动学习功能已关闭（在上方菜单栏，超星学习通-课程学习中开启）。${jobName} 即将跳过`;
                        easy_us_1.$message.warn({ content: msg });
                        background_1.$console.warn(msg);
                    }
                    else {
                        // 重复学习，或者未完成
                        if (opts.restudy || attachment.job) {
                            func = () => {
                                const msg = `即将${opts.restudy ? '重新' : ''}播放 : ` + jobName;
                                easy_us_1.$message.info({ content: msg });
                                background_1.$console.log(msg);
                                return mediaTask(opts, media, doc);
                            };
                        }
                    }
                }
                else if (chapterTest) {
                    if (!exports.CXProject.scripts.study.cfg.enableChapterTest) {
                        const msg = `章节测试自动答题功能已关闭（在上方菜单栏，超星学习通-课程学习中开启）。${jobName} 即将跳过`;
                        easy_us_1.$message.warn({ content: msg });
                        background_1.$console.warn(msg);
                    }
                    else {
                        const status = win.document.querySelector('.testTit_status');
                        // 已完成
                        if (status === null || status === void 0 ? void 0 : status.classList.contains('testTit_status_complete')) {
                            const msg = `章节测试已完成 : ` + jobName;
                            easy_us_1.$message.success({ content: msg });
                            background_1.$console.log(msg);
                        }
                        else {
                            if (attachment.job || common_1.CommonProject.scripts.settings.cfg['work-when-no-job']) {
                                func = () => {
                                    const msg = `开始答题 : ` + jobName;
                                    easy_us_1.$message.info({ content: msg });
                                    background_1.$console.log(msg);
                                    return chapterTestTask(root, opts.workOptions);
                                };
                            }
                            if (attachment.job === undefined && common_1.CommonProject.scripts.settings.cfg['work-when-no-job'] === false) {
                                const msg = `当前作业 ${jobName} 不是任务点，但待完成，如需开启自动答题请前往：通用-全局设置，开启强制答题。`;
                                easy_us_1.$message.warn({ content: msg });
                                background_1.$console.warn(msg);
                            }
                        }
                    }
                }
                else if (read) {
                    if (!exports.CXProject.scripts.study.cfg.enablePPT) {
                        const msg = `PPT/书籍阅读功能已关闭（在上方菜单栏，超星学习通-课程学习中开启）。${jobName} 即将跳过`;
                        easy_us_1.$message.warn({ content: msg });
                        background_1.$console.warn(msg);
                    }
                    else {
                        if (attachment.job) {
                            func = () => {
                                const msg = `正在学习 : ` + jobName;
                                easy_us_1.$message.warn({ content: msg });
                                background_1.$console.log(msg);
                                return readTask(win);
                            };
                        }
                    }
                }
                const job = {
                    mid: attachment.property.mid,
                    attachment: attachment,
                    func: func
                };
                searchedJobs.push(job);
                return job;
            }
        }
        else {
            return undefined;
        }
    };
    let job;
    for (const iframe of searchIFrame(knowCardWin.document)) {
        job = search(iframe);
        if (job) {
            return job;
        }
    }
    return job;
}
/**
 * 永久固定显示视频进度
 */
function fixedVideoProgress() {
    if (state.study.videojs) {
        const { bar } = (0, core_1.domSearch)({ bar: '.vjs-control-bar' }, state.study.videojs);
        if (bar) {
            bar.style.opacity = '1';
        }
    }
}
exports.fixedVideoProgress = fixedVideoProgress;
/**
 * 播放视频和音频
 */
function mediaTask(setting, media, doc) {
    return __awaiter(this, void 0, void 0, function* () {
        const { playbackRate = 1, volume = 0 } = setting;
        // @ts-ignore
        const { videojs } = (0, core_1.domSearch)({ videojs: '#video,#audio' }, doc);
        if (!videojs) {
            background_1.$console.error('视频检测不到，请尝试刷新或者手动切换下一章。');
            return;
        }
        state.study.videojs = videojs;
        // @ts-ignore
        top.currentMedia = media;
        // 固定视频进度
        fixedVideoProgress();
        // eslint-disable-next-line no-undef
        let reloadInterval;
        if (setting.reloadVideoWhenError) {
            reloadInterval = setInterval(() => {
                if (doc.documentElement.innerText.includes('网络错误导致视频下载中途失败')) {
                    background_1.$console.error('视频加载失败，即将刷新页面');
                    setTimeout(() => {
                        location.reload();
                    }, 3000);
                }
            }, 5000);
        }
        // 随机作答视频内题目
        if (setting.videoQuizStrategy === 'random') {
            const loop = () => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const submitBtn = () => doc.querySelector('#videoquiz-submit');
                if (submitBtn()) {
                    const list = Array.from(doc.querySelectorAll('.ans-videoquiz-opt label'));
                    const answer = list[Math.floor(Math.random() * list.length)];
                    answer === null || answer === void 0 ? void 0 : answer.click();
                    (_a = submitBtn()) === null || _a === void 0 ? void 0 : _a.click();
                    yield core_1.$.sleep(3000);
                    // 隐藏视频内题目元素
                    const container = doc.querySelector('#video .ans-videoquiz');
                    const components = Array.from(doc.querySelectorAll('.x-component-default'));
                    if (container) {
                        container.remove();
                    }
                    if (components.length) {
                        for (const com of components) {
                            com.style.display = 'none';
                        }
                    }
                }
                yield core_1.$.sleep(3000);
                yield loop();
            });
            loop();
        }
        /**
         * 视频播放
         */
        yield new Promise((resolve, reject) => {
            const playFunction = () => __awaiter(this, void 0, void 0, function* () {
                yield waitForFaceRecognition();
                if (!media.ended) {
                    yield core_1.$.sleep(1000);
                    media.play();
                    media.playbackRate = playbackRate;
                }
            });
            media.addEventListener('pause', playFunction);
            media.addEventListener('ended', () => {
                background_1.$console.log('视频播放完毕');
                media.removeEventListener('pause', playFunction);
                clearInterval(reloadInterval);
                resolve();
            });
            background_1.$console.log('视频开始播放');
            media.volume = volume;
            // 重置视频进度
            media.currentTime = 0;
            // 使用 setTimeout 解决 The play() request was interrupted by a call to pause() 问题
            setTimeout(() => {
                (0, utils_1.playMedia)(() => media.play())
                    .then(() => {
                    media.playbackRate = playbackRate;
                })
                    .catch(reject);
            }, 200);
        });
    });
}
/**
 * 阅读 ppt
 */
function readTask(win) {
    return __awaiter(this, void 0, void 0, function* () {
        const finishJob = win.finishJob;
        if (finishJob)
            finishJob();
        yield core_1.$.sleep(3000);
    });
}
/**
 * 章节测验
 */
function chapterTestTask(frame, { answererWrappers, period, upload, thread, stopSecondWhenFinish, redundanceWordsText, answerSeparators, answerMatchMode }) {
    return __awaiter(this, void 0, void 0, function* () {
        if (answererWrappers === undefined || answererWrappers.length === 0) {
            const msg = '检测到题库配置为空，无法自动答题，请前往 “通用-全局设置” 页面进行配置。';
            easy_us_1.$message.error({ content: msg });
            return background_1.$console.warn(msg);
        }
        easy_us_1.$message.info({
            content: (0, easy_us_1.h)('div', ['正在答题中，答题结果请前往：通用-搜索结果 进行查看']),
            duration: 10
        });
        background_1.$console.info('开始章节测试');
        const frameWindow = frame.contentWindow;
        const { TiMu } = (0, core_1.domSearchAll)({ TiMu: '.TiMu' }, frameWindow.document);
        common_1.CommonProject.scripts.workResults.methods.init();
        const chapterTestTaskQuestionTitleTransform = (titles) => {
            const transformed = core_1.StringUtils.of(titles.map((t) => (t ? (0, work_1.optimizationElementWithImage)(t).innerText : '')).join(','))
                .nowrap()
                .nospace()
                .toString()
                .trim()
                /** 超星旧版作业题目冗余数据 */
                .replace(/^\d+[。、.]/, '')
                .replace(/（\d+.\d+分）/, '')
                .replace(/\(..题, .+?分\)/, '')
                .replace(/[[(【（](.+题|名词解释|完形填空|阅读理解)[\])】）]/, '')
                .trim();
            return (0, work_1.removeRedundantWords)(transformed, redundanceWordsText.split('\n'));
        };
        /** 新建答题器 */
        const worker = new core_1.OCSWorker({
            root: TiMu,
            elements: {
                title: '.Zy_TItle .clearfix',
                /**
                 * 兼容各种选项
                 *
                 * ul li .after 单选多选
                 * ul li label:not(.after) 判断题
                 * ul li textarea 填空题
                 */
                options: 'ul li .after,ul li textarea,ul textarea,ul li label:not(.before)',
                type: 'input[id^="answertype"]',
                lineAnswerInput: '.line_answer input[name^=answer]',
                lineSelectBox: '.line_answer_ct .selectBox '
            },
            thread: thread !== null && thread !== void 0 ? thread : 1,
            answerSeparators: answerSeparators.split(',').map((s) => s.trim()),
            answerMatchMode: answerMatchMode,
            /** 默认搜题方法构造器 */
            answerer: (elements, ctx) => {
                const title = chapterTestTaskQuestionTitleTransform(elements.title);
                if (title) {
                    const typeInput = elements.type[0];
                    return common_1.CommonProject.scripts.apps.methods.searchAnswerInCaches(title, () => __awaiter(this, void 0, void 0, function* () {
                        yield core_1.$.sleep((period !== null && period !== void 0 ? period : 3) * 1000);
                        return (0, core_1.defaultAnswerWrapperHandler)(answererWrappers, {
                            type: (typeInput ? getQuestionType(parseInt(typeInput.value)) : undefined) || 'unknown',
                            title,
                            options: ctx.elements.options.map((o) => o.innerText).join('\n')
                        });
                    }));
                }
                else {
                    throw new Error('题目为空，请查看题目是否为空，或者忽略此题');
                }
            },
            work: (ctx) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const { elements, searchInfos } = ctx;
                const typeInput = elements.type[0];
                const type = typeInput ? getQuestionType(parseInt(typeInput.value)) : undefined;
                if (type && (type === 'completion' || type === 'multiple' || type === 'judgement' || type === 'single')) {
                    const resolver = (0, core_1.defaultQuestionResolve)(ctx)[type];
                    const handler = (type, answer, option, ctx) => {
                        var _a, _b, _c, _d, _e, _f, _g;
                        if (type === 'judgement' || type === 'single' || type === 'multiple') {
                            // 检查是否已经选择
                            const checked = ((_b = (_a = option === null || option === void 0 ? void 0 : option.parentElement) === null || _a === void 0 ? void 0 : _a.querySelector('label input')) === null || _b === void 0 ? void 0 : _b.getAttribute('checked')) === 'checked' ||
                                // 适配2023/9月最新版本
                                ((_c = option === null || option === void 0 ? void 0 : option.parentElement) === null || _c === void 0 ? void 0 : _c.getAttribute('aria-checked')) === 'true';
                            if (checked) {
                                // 跳过
                            }
                            else {
                                option === null || option === void 0 ? void 0 : option.click();
                            }
                        }
                        else if (type === 'completion' && answer.trim()) {
                            const text = (_d = option === null || option === void 0 ? void 0 : option.parentElement) === null || _d === void 0 ? void 0 : _d.querySelector('textarea');
                            const textareaFrame = (_e = option === null || option === void 0 ? void 0 : option.parentElement) === null || _e === void 0 ? void 0 : _e.querySelector('iframe');
                            if (text) {
                                text.value = answer;
                            }
                            if (textareaFrame === null || textareaFrame === void 0 ? void 0 : textareaFrame.contentDocument) {
                                textareaFrame.contentDocument.body.innerHTML = answer;
                            }
                            if ((_f = option === null || option === void 0 ? void 0 : option.parentElement) === null || _f === void 0 ? void 0 : _f.parentElement) {
                                /** 如果存在保存按钮则点击 */
                                (_g = (0, easy_us_1.$el)('[onclick*=saveQuestion]', option.parentElement.parentElement)) === null || _g === void 0 ? void 0 : _g.click();
                            }
                        }
                    };
                    return yield resolver(searchInfos, elements.options.map((option) => (0, work_1.optimizationElementWithImage)(option)), handler);
                }
                // 连线题自定义处理
                else if (type && type === 'line') {
                    for (const answers of searchInfos.map((info) => info.results.map((res) => res.answer))) {
                        let ans = answers;
                        if (ans.length === 1) {
                            ans = (0, core_1.splitAnswer)(ans[0]);
                        }
                        if (ans.filter(Boolean).length !== 0 && elements.lineAnswerInput) {
                            //  选择答案
                            for (let index = 0; index < elements.lineSelectBox.length; index++) {
                                const box = elements.lineSelectBox[index];
                                if (ans[index]) {
                                    (_a = (0, easy_us_1.$el)(`li[data=${ans[index]}] a`, box)) === null || _a === void 0 ? void 0 : _a.click();
                                    yield core_1.$.sleep(200);
                                }
                            }
                            return { finish: true };
                        }
                    }
                    return { finish: false };
                }
                return { finish: false };
            }),
            /** 完成答题后 */
            onResultsUpdate(curr, _, res) {
                var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
                return __awaiter(this, void 0, void 0, function* () {
                    common_1.CommonProject.scripts.workResults.methods.setResults((0, work_1.simplifyWorkResult)(res, chapterTestTaskQuestionTitleTransform));
                    if ((_a = curr.result) === null || _a === void 0 ? void 0 : _a.finish) {
                        common_1.CommonProject.scripts.apps.methods.addQuestionCacheFromWorkResult((0, work_1.simplifyWorkResult)([curr], chapterTestTaskQuestionTitleTransform));
                    }
                    common_1.CommonProject.scripts.workResults.methods.updateWorkStateByResults(res);
                    // 没有完成时随机作答
                    if (((_b = curr.result) === null || _b === void 0 ? void 0 : _b.finish) === false && curr.resolved === true) {
                        const options = ((_d = (_c = curr.ctx) === null || _c === void 0 ? void 0 : _c.elements) === null || _d === void 0 ? void 0 : _d.options) || [];
                        const typeInput = (_f = (_e = curr.ctx) === null || _e === void 0 ? void 0 : _e.elements) === null || _f === void 0 ? void 0 : _f.type[0];
                        const type = typeInput ? getQuestionType(parseInt(typeInput.value)) : undefined;
                        const commonSetting = common_1.CommonProject.scripts.settings.cfg;
                        if (commonSetting['randomWork-choice'] && (type === 'judgement' || type === 'single' || type === 'multiple')) {
                            background_1.$console.log('正在随机作答');
                            const option = options[Math.floor(Math.random() * options.length)];
                            // @ts-ignore 随机选择选项
                            (_h = (_g = option === null || option === void 0 ? void 0 : option.parentElement) === null || _g === void 0 ? void 0 : _g.querySelector('a,label')) === null || _h === void 0 ? void 0 : _h.click();
                        }
                        else if (commonSetting['randomWork-complete'] && type === 'completion') {
                            background_1.$console.log('正在随机作答');
                            // 随机填写答案
                            for (const option of options) {
                                const textarea = (_j = option === null || option === void 0 ? void 0 : option.parentElement) === null || _j === void 0 ? void 0 : _j.querySelector('textarea');
                                const completeTexts = commonSetting['randomWork-completeTexts-textarea'].split('\n').filter(Boolean);
                                const text = completeTexts[Math.floor(Math.random() * completeTexts.length)];
                                const textareaFrame = (_k = option === null || option === void 0 ? void 0 : option.parentElement) === null || _k === void 0 ? void 0 : _k.querySelector('iframe');
                                if (text) {
                                    if (textarea) {
                                        textarea.value = text;
                                    }
                                    if (textareaFrame === null || textareaFrame === void 0 ? void 0 : textareaFrame.contentDocument) {
                                        textareaFrame.contentDocument.body.innerHTML = text;
                                    }
                                }
                                else {
                                    background_1.$console.error('请设置随机填空的文案');
                                }
                                yield core_1.$.sleep(500);
                            }
                        }
                    }
                });
            },
            onElementSearched(elements) {
                return __awaiter(this, void 0, void 0, function* () {
                    const typeInput = elements.type[0];
                    const type = typeInput ? getQuestionType(parseInt(typeInput.value)) : undefined;
                    /** 判断题转换成文字，以便于答题程序判断 */
                    if (type === 'judgement') {
                        elements.options.forEach((option) => {
                            var _a, _b, _c;
                            if (option.innerText.includes('对') || option.innerText.includes('错')) {
                                // 2023/8/5日后超星已修复判断题，将图片修改成文字，如果已经有对错的文本，则不需要再转换
                            }
                            // 如果是英语的对错题目，他是一个英文单词 True,False
                            else if (((_a = option === null || option === void 0 ? void 0 : option.textContent) === null || _a === void 0 ? void 0 : _a.trim()) === 'True') {
                                if (((_b = option === null || option === void 0 ? void 0 : option.textContent) === null || _b === void 0 ? void 0 : _b.trim()) === 'True') {
                                    option.textContent = '√';
                                }
                                if (((_c = option === null || option === void 0 ? void 0 : option.textContent) === null || _c === void 0 ? void 0 : _c.trim()) === 'False') {
                                    option.textContent = 'x';
                                }
                            }
                            else {
                                const ri = option.querySelector('.ri');
                                const span = document.createElement('span');
                                span.innerText = ri ? '√' : '×';
                                option.appendChild(span);
                            }
                        });
                    }
                });
            }
        });
        const results = yield worker.doWork();
        const msg = `答题完成，将等待 ${stopSecondWhenFinish} 秒后进行保存或提交。`;
        background_1.$console.info(msg);
        easy_us_1.$message.info({ content: msg, duration: stopSecondWhenFinish });
        yield core_1.$.sleep(stopSecondWhenFinish * 1000);
        // 处理提交
        yield worker.uploadHandler({
            type: upload,
            results,
            callback(finishedRate, uploadable) {
                return __awaiter(this, void 0, void 0, function* () {
                    const msg = `完成率 ${finishedRate.toFixed(2)} :  ${uploadable ? '3秒后将自动提交' : '3秒后将自动保存'} `;
                    background_1.$console.info(msg);
                    easy_us_1.$message.success({ content: msg, duration: 3 });
                    yield core_1.$.sleep(3000);
                    if (uploadable) {
                        // @ts-ignore 提交
                        frameWindow.btnBlueSubmit();
                        yield core_1.$.sleep(3000);
                        /** 确定按钮 */
                        // @ts-ignore 确定
                        frameWindow.submitCheckTimes();
                        // @ts-ignore 2024/4 更新后上方函数无法关闭弹窗，需要手动关闭确定弹窗
                        top.$('#workpop').hide();
                    }
                    else {
                        // @ts-ignore 禁止弹窗
                        frameWindow.alert = () => { };
                        // @ts-ignore 暂时保存
                        frameWindow.noSubmit();
                    }
                });
            }
        });
        worker.emit('done');
    });
}
/**
 * 获取有效的数字
 * @param nums
 */
function getValidNumber(...nums) {
    return nums.map((num) => (typeof num === 'number' ? num : undefined)).find((num) => num !== undefined);
}
exports.getValidNumber = getValidNumber;
/**
 * cx 题目类型 ：
 * 0 单选题
 * 1 多选题
 * 2 简答题
 * 3 判断题
 * 4 填空题
 * 5 名词解释
 * 6 论述题
 * 7 计算题
 * 8 其他题(大概率是填空题)
 * 9 分录题
 * 10 资料题
 * 11 连线题
 * 14 完形填空
 * 15 阅读理解
 */
function getQuestionType(val) {
    return val === 0
        ? 'single'
        : val === 1
            ? 'multiple'
            : val === 3
                ? 'judgement'
                : [2, 4, 5, 6, 7, 8, 9, 10].some((t) => t === val)
                    ? 'completion'
                    : val === 11
                        ? 'line'
                        : val === 14
                            ? 'fill'
                            : val === 15
                                ? 'reader'
                                : undefined;
}
/** 阅读理解和完形填空的共同处理器 */
function readerAndFillHandle(searchInfos, list) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        for (const answers of searchInfos.map((info) => info.results.map((res) => res.answer))) {
            let ans = answers;
            if (ans.length === 1) {
                ans = (0, core_1.splitAnswer)(ans[0]);
            }
            if (ans.filter(Boolean).length !== 0 && list.length !== 0) {
                for (let index = 0; index < ans.length; index++) {
                    const item = list[index];
                    if (item) {
                        /** 获取每个小题中的准确答案选项 并点击 */
                        (_a = (0, easy_us_1.$el)(`span.saveSingleSelect[data="${ans[index]}"]`, item)) === null || _a === void 0 ? void 0 : _a.click();
                        yield core_1.$.sleep(200);
                    }
                }
                return { finish: true };
            }
        }
        return { finish: false };
    });
}
function waitForFaceRecognition() {
    let notified = false;
    return new Promise((resolve) => {
        const interval = setInterval(() => {
            const face = (0, easy_us_1.$el)('#fcqrimg', top === null || top === void 0 ? void 0 : top.document);
            if (face) {
                if (!notified) {
                    notified = true;
                    if (exports.CXProject.scripts.study.cfg.notifyWhenHasFaceRecognition) {
                        common_1.CommonProject.scripts.settings.methods.notificationBySetting('检测到人脸识别，请手动进行识别后脚本才会继续运行。', { duration: 0 });
                    }
                    background_1.$console.warn('检测到人脸识别，请手动进行识别后脚本才会继续运行。');
                }
            }
            else {
                clearInterval(interval);
                resolve();
            }
        }, 3000);
    });
}
