"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ICourseProject = void 0;
const core_1 = require("@ocsjs/core");
const easy_us_1 = require("easy-us");
const utils_1 = require("../utils");
const common_1 = require("./common");
const work_1 = require("../utils/work");
const background_1 = require("./background");
const app_1 = require("../utils/app");
const study_1 = require("../utils/study");
const configs_1 = require("../utils/configs");
const state = {
    currentMedia: undefined,
    currentUrlHash: '',
    currentRunningScriptName: ''
};
exports.ICourseProject = easy_us_1.Project.create({
    name: '中国大学MOOC',
    domains: ['icourse163.org'],
    scripts: {
        dispatcher: new easy_us_1.Script({
            name: '调度器',
            hideInPanel: true,
            matches: [['所有页面', 'icourse163.org']],
            oncomplete() {
                setInterval(() => {
                    var _a, _b;
                    const hash = new URL(window.location.href).hash;
                    if (state.currentUrlHash !== hash) {
                        state.currentRunningScriptName = '';
                    }
                    state.currentUrlHash = hash;
                    for (const key in exports.ICourseProject.scripts) {
                        if (Object.prototype.hasOwnProperty.call(exports.ICourseProject.scripts, key)) {
                            const script = exports.ICourseProject.scripts[key];
                            const runAtHash = Array.isArray(script.cfg.runAtHash) ? script.cfg.runAtHash : [script.cfg.runAtHash];
                            if (runAtHash.length && runAtHash.some((h) => state.currentUrlHash.includes(h))) {
                                if (state.currentRunningScriptName !== script.name) {
                                    state.currentRunningScriptName = script.name;
                                    (_b = (_a = script.methods) === null || _a === void 0 ? void 0 : _a.main) === null || _b === void 0 ? void 0 : _b.call(_a, () => {
                                        return state.currentUrlHash && runAtHash.some((h) => state.currentUrlHash.includes(h));
                                    });
                                }
                                break;
                            }
                        }
                    }
                }, 1000);
            }
        }),
        guide: new easy_us_1.Script({
            name: '💡 使用提示',
            matches: [['', 'icourse163.org']],
            // 添加版本号是因为只有 notes 会强制更新，其他配置项不会，如果需要修改 runAtHash ，需要更新版本号
            namespace: 'icourse.guide-v1',
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes(['手动进入任意课程里的课件/作业，即可开始自动学习']).outerHTML
                },
                runAtHash: {
                    // 在没有进入学习页面前，都显示提示
                    defaultValue: ['/home/course']
                }
            },
            methods() {
                return {
                    main: () => __awaiter(this, void 0, void 0, function* () {
                        console.log(state, this.cfg.runAtHash);
                        common_1.CommonProject.scripts.render.methods.pin(this);
                    })
                };
            }
        }),
        study: new easy_us_1.Script({
            name: '🖥️ 学习脚本',
            // 添加版本号是因为只有 notes 会强制更新，其他配置项不会，如果需要修改 runAtHash ，需要更新版本号
            namespace: 'icourse.study-v1',
            matches: [
                ['MOOC作业页面', 'icourse163.org/learn'],
                ['SPOC作业页面', 'icourse163.org/spoc/learn']
            ],
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        '请勿在使用过程中最小化浏览器',
                        '自动讨论默认关闭，如需开启请在下方设置中设置',
                        '作业请完成课程后手动进入'
                    ]).outerHTML
                },
                runAtHash: {
                    defaultValue: '/learn/content?type=detail'
                },
                playbackRate: configs_1.playbackRate,
                volume: configs_1.volume,
                readSpeed: {
                    label: 'PPT翻阅速度（秒）',
                    attrs: { type: 'number', step: '1', min: '1', max: '10' },
                    defaultValue: 1
                },
                discussionStrategy: {
                    label: '讨论自动回复方式',
                    tag: 'select',
                    defaultValue: 'not-reply',
                    options: [
                        ['not-reply', '不讨论回复'],
                        ['max-show-up', '获取出现最多的评论进行回复'],
                        ['max-fav', '获取最多点赞的评论进行回复'],
                        ['use-newest', '获取最新的评论进行回复']
                    ]
                },
                enableChapterTest: {
                    label: '随堂测验自动答题',
                    attrs: {
                        type: 'checkbox',
                        title: '是否开启随堂测验自动答题，默认关闭，测试时只需点击即可完成测验，但这里保留选项防止需要开启。'
                    },
                    defaultValue: false
                }
            },
            oncomplete() {
                this.onConfigChange('playbackRate', (playbackRate) => {
                    state.currentMedia && (state.currentMedia.playbackRate = parseFloat(playbackRate.toString()));
                });
                this.onConfigChange('volume', (v) => state.currentMedia && (state.currentMedia.volume = v));
            },
            methods() {
                return {
                    main: (canRun) => __awaiter(this, void 0, void 0, function* () {
                        common_1.CommonProject.scripts.render.methods.pin(this);
                        const remotePage = yield core_1.RemotePlaywright.getCurrentPage();
                        // 检查是否为软件环境
                        if (!remotePage) {
                            return app_1.$playwright.showError();
                        }
                        // 移动窗口到边缘
                        common_1.CommonProject.scripts.render.methods.moveToEdge();
                        const study = () => __awaiter(this, void 0, void 0, function* () {
                            var _a, _b;
                            const lessonName = (_a = document.querySelector('.j-lesson .j-up')) === null || _a === void 0 ? void 0 : _a.textContent;
                            const currentUnitItem = document.querySelector('.j-unitslist  li.current');
                            const unitName = (_b = currentUnitItem === null || currentUnitItem === void 0 ? void 0 : currentUnitItem.querySelector('.unit-name')) === null || _b === void 0 ? void 0 : _b.textContent;
                            background_1.$console.log(`正在学习：${lessonName || ''} - ${unitName || ''}`);
                            const isJob = (iconName) => currentUnitItem === null || currentUnitItem === void 0 ? void 0 : currentUnitItem.querySelector(`[class*=${iconName}]`);
                            let hasJob = true;
                            if (isJob('u-icon-video')) {
                                yield (0, study_1.waitForElement)('video, audio');
                                yield watchMedia(this.cfg.playbackRate, this.cfg.volume);
                                background_1.$console.log('视频学习完成');
                            }
                            else if (isJob('u-icon-doc')) {
                                yield (0, study_1.waitForElement)('.ux-pdf-reader');
                                yield readPPT(remotePage, this.cfg.readSpeed);
                                background_1.$console.log('PPT完成');
                            }
                            else if (isJob('u-icon-discuss')) {
                                yield (0, study_1.waitForElement)('.j-reply-all');
                                yield discussion(remotePage, this.cfg.discussionStrategy);
                                background_1.$console.log('讨论完成');
                            }
                            else if (isJob('u-icon-test')) {
                                const replay = yield (0, study_1.waitForElement)('.j-replay');
                                if ((replay === null || replay === void 0 ? void 0 : replay.style.display) === 'none') {
                                    if (this.cfg.enableChapterTest) {
                                        yield new Promise((resolve) => {
                                            exports.ICourseProject.scripts.work.methods.start('chapter-test', canRun, (worker) => {
                                                console.log('worker', worker);
                                                worker.once('done', resolve);
                                                worker.once('close', resolve);
                                            });
                                        });
                                        background_1.$console.log('测验完成');
                                    }
                                    else {
                                        background_1.$console.warn('随堂测验自动答题功能已关闭（上方菜单栏-中国大学MOOC-学习脚本中开启），即将跳过。');
                                    }
                                }
                                else {
                                    background_1.$console.log('随堂测验已完成，即将跳过。');
                                }
                            }
                            else if (isJob('u-icon-text')) {
                                // 文档无需处理
                                background_1.$console.log('文档无需处理，即将跳过。');
                            }
                            else {
                                hasJob = false;
                            }
                            yield core_1.$.sleep(3000);
                            // 跳转下一章，然后通过URL变化，调度器会重新执行此 main 函数
                            if (canRun()) {
                                if (hasJob) {
                                    background_1.$console.log('准备跳转下一章');
                                }
                                else {
                                    background_1.$console.warn('未找到学习内容，或者此章节不支持自动学习！即将跳过本章节');
                                }
                                yield gotoNextJob();
                            }
                        });
                        study();
                        function gotoNextJob() {
                            return __awaiter(this, void 0, void 0, function* () {
                                const list = yield next();
                                for (const item of list) {
                                    const el = typeof item === 'function' ? item() : item;
                                    if (el) {
                                        yield (remotePage === null || remotePage === void 0 ? void 0 : remotePage.click(el));
                                    }
                                }
                                if (list.length === 0) {
                                    easy_us_1.$message.success({ content: '所有章节学习完成！', duration: 0 });
                                    background_1.$console.info('所有章节学习完成！');
                                    common_1.CommonProject.scripts.settings.methods.notificationBySetting('所有章节学习完成！', {
                                        duration: 0,
                                        extraTitle: '中国大学MOOC学习脚本'
                                    });
                                }
                            });
                        }
                        function next() {
                            var _a;
                            return __awaiter(this, void 0, void 0, function* () {
                                const nextEl = (_a = document.querySelector('.unitslist .current')) === null || _a === void 0 ? void 0 : _a.nextElementSibling;
                                // 判断小节
                                if (nextEl) {
                                    return [nextEl.querySelector('.unit-name')];
                                }
                                // 判断章节
                                const getName = (node) => { var _a; return (_a = node === null || node === void 0 ? void 0 : node.textContent) === null || _a === void 0 ? void 0 : _a.replace(/\s/g, ''); };
                                const lessonName = getName(document.querySelector('.j-lesson .j-up'));
                                if (!lessonName) {
                                    throw Error('无法读取章节名!');
                                }
                                const lessonList = Array.from(document.querySelectorAll('.j-lesson .j-list .list'));
                                let nextLesson = undefined;
                                for (const item of lessonList) {
                                    const itemName = getName(item);
                                    if (itemName === lessonName) {
                                        if (item.nextElementSibling) {
                                            nextLesson = item.nextElementSibling;
                                            break;
                                        }
                                    }
                                }
                                if (nextLesson) {
                                    return [
                                        // 点击展开章节列表
                                        document.querySelector('.j-lesson'),
                                        // 点击章节
                                        nextLesson
                                    ];
                                }
                                // 判断单元
                                const chapterName = getName(document.querySelector('.j-chapter .j-up'));
                                if (!chapterName) {
                                    throw Error('无法读取单元名!');
                                }
                                const chapterList = Array.from(document.querySelectorAll('.j-chapter .j-list .list'));
                                let nextChapter = undefined;
                                for (const item of chapterList) {
                                    const itemName = getName(item);
                                    if (itemName === chapterName) {
                                        if (item.nextElementSibling) {
                                            nextChapter = item.nextElementSibling;
                                            break;
                                        }
                                    }
                                }
                                if (nextChapter) {
                                    return [
                                        // 点击展开单元列表
                                        () => document.querySelector('.j-chapter'),
                                        // 点击单元
                                        () => nextChapter,
                                        // 点击展开章节列表
                                        () => document.querySelector('.j-lesson'),
                                        // 点击第一个章节
                                        () => document.querySelectorAll('.j-lesson .j-list .list')[0]
                                    ];
                                }
                                else {
                                    return [];
                                }
                            });
                        }
                    })
                };
            }
        }),
        work: new easy_us_1.Script({
            name: '✍️ 作业脚本',
            // 添加版本号是因为只有 notes 会强制更新，其他配置项不会，如果需要修改 runAtHash ，需要更新版本号
            namespace: 'icourse.work-v1',
            matches: [
                ['MOOC作业页面', 'icourse163.org/learn'],
                ['SPOC作业页面', 'icourse163.org/spoc/learn']
            ],
            configs: {
                notes: configs_1.workNotes,
                runAtHash: {
                    defaultValue: '/learn/quiz'
                }
            },
            methods() {
                const start = (type, canRun, onWorkerCreated) => __awaiter(this, void 0, void 0, function* () {
                    common_1.CommonProject.scripts.render.methods.pin(this);
                    // 移动窗口到边缘
                    common_1.CommonProject.scripts.render.methods.moveToEdge();
                    // 检查是否为软件环境
                    const remotePage = yield core_1.RemotePlaywright.getCurrentPage();
                    // 检查是否为软件环境
                    if (!remotePage) {
                        return app_1.$playwright.showError();
                    }
                    // 等待加载题目
                    yield waitForQuestion();
                    background_1.$console.log('开始答题');
                    common_1.CommonProject.scripts.render.methods.pin(this);
                    (0, work_1.commonWork)(this, {
                        workerProvider: (opts) => {
                            const worker = workAndExam(remotePage, type, opts);
                            worker.once('close', () => {
                                clearInterval(interval);
                            });
                            const interval = setInterval(() => {
                                if (canRun() === false) {
                                    easy_us_1.$message.warn('检测到页面切换，无法继续答题，将关闭自动答题。');
                                    clearInterval(interval);
                                    worker.emit('close');
                                }
                            }, 1000);
                            return worker;
                        },
                        onWorkerCreated: onWorkerCreated
                    });
                });
                return {
                    main: (canRun) => __awaiter(this, void 0, void 0, function* () {
                        if (location.hash.includes('learn/quizscore')) {
                            easy_us_1.$message.success('当前作业已完成，自动答题关闭。');
                            return;
                        }
                        return start('work-or-exam', canRun);
                    }),
                    start: start
                };
            }
        })
    }
});
function waitForQuestion() {
    return new Promise((resolve, reject) => {
        const interval = setInterval(() => {
            if (document.querySelector('.u-questionItem')) {
                clearInterval(interval);
                resolve();
            }
        }, 1000);
    });
}
function workAndExam(remotePage, type, { answererWrappers, period, thread, redundanceWordsText, upload, stopSecondWhenFinish, answerSeparators, answerMatchMode }) {
    common_1.CommonProject.scripts.workResults.methods.init({
        questionPositionSyncHandlerType: 'icourse'
    });
    const titleTransform = (titles) => {
        return (0, work_1.removeRedundantWords)(titles
            .map((t) => (t ? (0, work_1.optimizationElementWithImage)(t).innerText : ''))
            .filter((t) => t.trim() !== '')
            .join(',')
            // /\u200B/g 排除不可见的空格
            .replace(/[\u200A-\u200F]/g, ''), redundanceWordsText.split('\n'));
    };
    /** 新建答题器 */
    const worker = new core_1.OCSWorker({
        root: '.u-questionItem',
        elements: {
            title: '.j-title .j-richTxt',
            options: '.choices li,.inputArea'
        },
        thread: thread !== null && thread !== void 0 ? thread : 1,
        answerSeparators: answerSeparators.split(',').map((s) => s.trim()),
        answerMatchMode: answerMatchMode,
        /** 默认搜题方法构造器 */
        answerer: (elements, ctx) => {
            const title = titleTransform(elements.title);
            if (title) {
                return common_1.CommonProject.scripts.apps.methods.searchAnswerInCaches(title, () => __awaiter(this, void 0, void 0, function* () {
                    yield core_1.$.sleep((period !== null && period !== void 0 ? period : 3) * 1000);
                    return (0, core_1.defaultAnswerWrapperHandler)(answererWrappers, {
                        type: ctx.type || 'unknown',
                        title,
                        options: ctx.elements.options.map((o) => o.innerText).join('\n')
                    });
                }));
            }
            else {
                throw new Error('题目为空，请查看题目是否为空，或者忽略此题');
            }
        },
        work: {
            /** 自定义处理器 */
            handler(type, answer, option) {
                return __awaiter(this, void 0, void 0, function* () {
                    if (type === 'judgement' || type === 'single' || type === 'multiple') {
                        const text = option.querySelector('.f-richEditorText');
                        const input = option.querySelector('input');
                        if (input && !(input === null || input === void 0 ? void 0 : input.checked) && text) {
                            yield remotePage.click(text);
                        }
                    }
                    else if (type === 'completion' && answer.trim()) {
                        const text = option.querySelector('textarea');
                        if (text) {
                            text.value = answer.trim();
                            yield remotePage.click(text);
                        }
                    }
                });
            }
        },
        onElementSearched(elements, root) {
            elements.options.forEach((el) => {
                const correct = el.querySelector('.u-icon-correct');
                const wrong = el.querySelector('.u-icon-wrong');
                if (correct) {
                    correct.replaceWith('对');
                }
                if (wrong) {
                    wrong.replaceWith('错');
                }
            });
        },
        /** 完成答题后 */
        onResultsUpdate(curr, _, res) {
            var _a;
            common_1.CommonProject.scripts.workResults.methods.setResults((0, work_1.simplifyWorkResult)(res, titleTransform));
            if ((_a = curr.result) === null || _a === void 0 ? void 0 : _a.finish) {
                common_1.CommonProject.scripts.apps.methods.addQuestionCacheFromWorkResult((0, work_1.simplifyWorkResult)([curr], titleTransform));
            }
            common_1.CommonProject.scripts.workResults.methods.updateWorkStateByResults(res);
        }
    });
    worker
        .doWork()
        .then((results) => __awaiter(this, void 0, void 0, function* () {
        if (worker.isClose) {
            return;
        }
        if (type === 'chapter-test') {
            easy_us_1.$message.info(`答题完成，将等待 ${stopSecondWhenFinish} 秒后进行保存或提交。`);
            background_1.$console.info(`答题完成，将等待 ${stopSecondWhenFinish} 秒后进行保存或提交。`);
            yield core_1.$.sleep(stopSecondWhenFinish * 1000);
            if (worker.isClose) {
                return;
            }
            // 处理提交
            yield worker.uploadHandler({
                type: upload,
                results,
                callback(finishedRate, uploadable) {
                    return __awaiter(this, void 0, void 0, function* () {
                        const content = `完成率 ${finishedRate.toFixed(2)} :  ${uploadable ? '3秒后将自动提交' : '3秒后将自动跳过（没保存按钮）'} `;
                        background_1.$console.info(content);
                        easy_us_1.$message.success({ content: content, duration: 0 });
                        yield core_1.$.sleep(3000);
                        if (worker.isClose) {
                            return;
                        }
                        if (uploadable) {
                            const sumbit = document.querySelector('.j-submit');
                            if (sumbit) {
                                yield remotePage.click(sumbit);
                            }
                            else {
                                background_1.$console.warn('没有找到提交按钮，将跳过提交。');
                            }
                        }
                    });
                }
            });
        }
        else {
            easy_us_1.$message.success({ content: '作业/考试完成，请自行检查后保存或提交。', duration: 0 });
        }
        worker.emit('done');
    }))
        .catch((err) => {
        easy_us_1.$message.error({ content: '答题程序发生错误 : ' + err.message, duration: 0 });
    });
    return worker;
}
function watchMedia(playbackRate, volume) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            // 部分用户视频加载很慢，这里等待一下
            (0, study_1.waitForMedia)()
                .then((video) => {
                video.playbackRate = playbackRate;
                video.volume = volume;
                state.currentMedia = video;
                (0, utils_1.playMedia)(() => video === null || video === void 0 ? void 0 : video.play());
                video.onpause = () => __awaiter(this, void 0, void 0, function* () {
                    if (!(video === null || video === void 0 ? void 0 : video.ended)) {
                        video === null || video === void 0 ? void 0 : video.play();
                    }
                });
                video.onended = () => {
                    resolve();
                };
            })
                .catch(reject);
        });
    });
}
function readPPT(remotePage, readSpeed) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const reader = document.querySelector('.ux-pdf-reader');
        if (reader) {
            const total = parseInt(((_c = (_b = (_a = document
                .querySelector('.ux-h5pdfreader_container_footer_pages_total')) === null || _a === void 0 ? void 0 : _a.childNodes[1]) === null || _b === void 0 ? void 0 : _b.textContent) === null || _c === void 0 ? void 0 : _c.replace(/\s/, '')) || '0');
            const start = parseInt(((_d = document.querySelector('.ux-h5pdfreader_container_footer_pages_in')) === null || _d === void 0 ? void 0 : _d.value) || '1');
            for (let index = start; index < total + 1; index++) {
                const next = document.querySelector('.ux-h5pdfreader_container_footer_pages_next');
                if (next) {
                    yield remotePage.click(next);
                }
                else {
                    background_1.$console.error('未找到PPT的下一页按钮！');
                }
                yield core_1.$.sleep(readSpeed * 1000);
            }
        }
    });
}
function discussion(remotePage, discussionStrategy) {
    var _a, _b, _c, _d, _e, _f, _g;
    return __awaiter(this, void 0, void 0, function* () {
        if (discussionStrategy === 'not-reply') {
            return background_1.$console.warn('讨论自动回复功能已关闭（上方菜单栏-中国大学MOOC-学习脚本中开启）。');
        }
        let res = '';
        if (discussionStrategy === 'max-show-up') {
            const list = Array.from(document.querySelectorAll('.j-reply-all .f-pr .j-content'));
            const mapping = new Map();
            for (const item of list) {
                mapping.set(item.textContent, (mapping.get(item.textContent) || 0) + 1);
            }
            const content = (_b = (_a = [...mapping.entries()].sort((a, b) => b[1] - a[1])) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b[0];
            if (!content) {
                background_1.$console.error('读取出现最多评论失败！');
            }
            res = content;
        }
        else if (discussionStrategy === 'max-fav') {
            const list = Array.from(document.querySelectorAll('.j-reply-all .f-pr'));
            let max = 0;
            let maxEl = undefined;
            for (const item of list) {
                const num = parseInt(((_c = item.querySelector('.bar .num')) === null || _c === void 0 ? void 0 : _c.textContent) || '0');
                if (num > max) {
                    max = num;
                    maxEl = item;
                }
            }
            const content = ((_d = maxEl === null || maxEl === void 0 ? void 0 : maxEl.querySelector('.j-content')) === null || _d === void 0 ? void 0 : _d.textContent) || '';
            if (!content) {
                background_1.$console.error('读取最多点赞评论失败！');
            }
            res = content;
        }
        else if (discussionStrategy === 'use-newest') {
            const content = ((_e = document.querySelector('.j-reply-all .f-pr .first .j-content')) === null || _e === void 0 ? void 0 : _e.textContent) || '';
            if (!content) {
                background_1.$console.error('读取最新评论失败！');
            }
            res = content;
        }
        const p = (_g = (_f = document
            .querySelector('[id*=ueditor]')) === null || _f === void 0 ? void 0 : _f.contentDocument) === null || _g === void 0 ? void 0 : _g.querySelector('body p');
        if (p) {
            p.innerText = res;
            yield core_1.$.sleep(1000);
            const submit = document.querySelector('.ui-richEditor .u-btn-sm');
            if (submit) {
                yield remotePage.click(submit);
            }
            else {
                background_1.$console.error('提交按钮获取失败！');
            }
            yield core_1.$.sleep(2000);
        }
        else {
            background_1.$console.error('获取评论输入框失败！');
        }
    });
}
