"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IcveMoocProject = void 0;
const core_1 = require("@ocsjs/core");
const easy_us_1 = require("easy-us");
const configs_1 = require("../utils/configs");
const utils_1 = require("../utils");
const common_1 = require("./common");
const work_1 = require("../utils/work");
const background_1 = require("./background");
const study_1 = require("../utils/study");
const state = {
    study: {
        currentMedia: undefined,
        currentStudyLockId: 0,
        playbackRateWarningListenerId: 0,
        courseLengthListenerId: 0
    }
};
/**
 * 学习锁，用于判断是否可以学习，防止学习函数被多次调用
 */
class StudyLock {
    constructor() {
        StudyLock.auto_inc++;
        this.id = StudyLock.auto_inc;
        state.study.currentStudyLockId = this.id;
    }
    canStudy() {
        return this.id === state.study.currentStudyLockId;
    }
    static getLock() {
        return new StudyLock();
    }
}
StudyLock.auto_inc = 0;
exports.IcveMoocProject = easy_us_1.Project.create({
    name: '智慧职教',
    domains: [
        'icve.com.cn',
        'course.icve.com.cn',
        // 智慧职教套壳
        'courshare.cn'
    ],
    scripts: {
        guide: new easy_us_1.Script({
            name: '💡 使用提示',
            matches: [
                ['个人首页', 'icve.com.cn/studycenter'],
                ['学习页面', 'icve.com.cn/study/directory'],
                ['MOOC学院-个人首页', 'user.icve.com.cn'],
                ['MOOC学院-首页', 'mooc.icve.com.cn']
            ],
            namespace: 'icve.guide',
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes(['请点击任意课程进入', '进入课程后点击任意章节进入，即可自动学习']).outerHTML
                }
            },
            oncomplete() {
                common_1.CommonProject.scripts.render.methods.pin(this);
            }
        }),
        /** 智慧职教学习中心 */
        studyCenter: new easy_us_1.Script({
            name: '🖥️ 智慧职教-学习中心',
            namespace: 'icve.study.center',
            matches: [
                ['学习中心页面', '/study/directory/dir_course.html'],
                ['课程列表', 'icve.com.cn/study/directory/directory_list.html']
            ],
            configs: {
                playbackRate: configs_1.playbackRate,
                volume: configs_1.volume,
                /** 章节列表 */
                currentCourseUrlList: {
                    defaultValue: []
                }
            },
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    if (location.href.includes('icve.com.cn/study/directory/directory_list.html')) {
                        yield (0, study_1.waitForElement)('.h_cells a');
                        this.cfg.currentCourseUrlList = Array.from(document.querySelectorAll('.h_cells a')).map((a) => a.href);
                        return;
                    }
                    if (this.cfg.currentCourseUrlList.length === 0) {
                        try {
                            const url = 'https://www.icve.com.cn/study/directory/directory_list.html?courseId=' +
                                new URL(location.href).searchParams.get('courseId');
                            const res = yield fetch(url).then((res) => res.text());
                            const doc = new DOMParser().parseFromString(res, 'text/html');
                            this.cfg.currentCourseUrlList = Array.from(doc.querySelectorAll('.h_cells a')).map((a) => a.href);
                        }
                        catch (e) {
                            console.error(e);
                            easy_us_1.$message.error('课程列表获取失败，请刷新页面重试。');
                            return;
                        }
                    }
                    common_1.CommonProject.scripts.render.methods.pin(this);
                    this.onConfigChange('playbackRate', (playbackRate) => {
                        state.study.currentMedia && (state.study.currentMedia.playbackRate = parseFloat(playbackRate.toString()));
                    });
                    this.onConfigChange('volume', (v) => state.study.currentMedia && (state.study.currentMedia.volume = v));
                    const study = () => __awaiter(this, void 0, void 0, function* () {
                        var _a;
                        const res = yield Promise.race([(0, study_1.waitForElement)('video, audio'), (0, study_1.waitForElement)('.docBox')]);
                        if (res) {
                            const jobName = ((_a = document.querySelector('.tabsel.seled')) === null || _a === void 0 ? void 0 : _a.getAttribute('title')) || '-';
                            easy_us_1.$message.info('开始任务：' + jobName);
                            background_1.$console.log(`任务 ${jobName} 开始。`);
                            if (document.querySelector('video, audio')) {
                                const media = yield (0, study_1.waitForMedia)();
                                state.study.currentMedia = media;
                                media.volume = this.cfg.volume;
                                yield new Promise((resolve, reject) => {
                                    try {
                                        console.log(document.hasFocus());
                                        window.focus();
                                        // @ts-ignore
                                        easy_us_1.$gm.unsafeWindow.jwplayer().onComplete(() => __awaiter(this, void 0, void 0, function* () {
                                            background_1.$console.log('视频/音频播放完成。');
                                            yield core_1.$.sleep(3000);
                                            resolve();
                                        }));
                                        const play = () => {
                                            easy_us_1.$gm.unsafeWindow.jwplayer().play();
                                            easy_us_1.$gm.unsafeWindow.jwplayer().play();
                                            media.playbackRate = parseFloat(this.cfg.playbackRate.toString());
                                        };
                                        media.addEventListener('pause', () => __awaiter(this, void 0, void 0, function* () {
                                            if (!media.ended) {
                                                yield core_1.$.sleep(1000);
                                                (0, utils_1.playMedia)(play);
                                            }
                                        }));
                                        // 开始播放
                                        (0, utils_1.playMedia)(play);
                                    }
                                    catch (err) {
                                        reject(err);
                                    }
                                });
                            }
                            easy_us_1.$message.success(`任务 ${jobName} 完成，三秒后下一章`);
                            background_1.$console.log(`任务 ${jobName} 完成，三秒后下一章`);
                        }
                        else {
                            background_1.$console.error(`不支持的任务页面，请跟作者进行反馈。三秒后下一章`);
                        }
                        yield core_1.$.sleep(3000);
                        next();
                    });
                    const next = () => {
                        for (let index = 0; index < this.cfg.currentCourseUrlList.length; index++) {
                            const url = this.cfg.currentCourseUrlList[index];
                            const nextUrl = this.cfg.currentCourseUrlList[index + 1];
                            if (new URL(url).hash === new URL(location.href).hash) {
                                if (!nextUrl) {
                                    easy_us_1.$modal.alert({ content: '全部任务已完成' });
                                    common_1.CommonProject.scripts.settings.methods.notificationBySetting('全部任务点已完成！', {
                                        duration: 0,
                                        extraTitle: '智慧职教学习脚本'
                                    });
                                    return;
                                }
                                else {
                                    window.location.href = this.cfg.currentCourseUrlList[index + 1];
                                }
                            }
                        }
                    };
                    study();
                });
            }
        }),
        /** MOOC 学院 */
        study: new easy_us_1.Script({
            name: '🖥️ MOOC学院-课程学习',
            namespace: 'icve.study.main',
            matches: [['课程学习页面', '/learnspace/learn/learn/templateeight/index.action']],
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        '如果视频无法播放，可以手动点击其他任务跳过视频。',
                        '经过测试视频倍速最多二倍，否则会判定无效。',
                        '手动进入作业页面才能使用自动答题。'
                    ]).outerHTML
                },
                playbackRate: configs_1.playbackRate,
                volume: configs_1.volume,
                restudy: configs_1.restudy,
                showScrollBar: {
                    label: '显示右侧滚动条',
                    attrs: { type: 'checkbox' },
                    defaultValue: true
                },
                expandAll: {
                    label: '展开所有章节',
                    attrs: { type: 'checkbox' },
                    defaultValue: true
                },
                switchPeriod: {
                    label: '下一章节切换间隔（秒）',
                    defaultValue: 10,
                    attrs: {
                        type: 'number',
                        min: 0,
                        max: 999,
                        step: 1
                    }
                }
            },
            onrender() {
                // 高倍速警告
                this.offConfigChange(state.study.playbackRateWarningListenerId);
                state.study.playbackRateWarningListenerId =
                    this.onConfigChange('playbackRate', (playbackRate) => {
                        if (playbackRate > 4) {
                            easy_us_1.$modal.alert({
                                title: '⚠️高倍速警告',
                                content: easy_us_1.$ui.notes(['高倍速可能导致视频无法完成！'])
                            });
                        }
                    }) || 0;
            },
            oncomplete() {
                var _a;
                return __awaiter(this, void 0, void 0, function* () {
                    common_1.CommonProject.scripts.render.methods.pin(this);
                    yield core_1.$.sleep(3000);
                    this.onConfigChange('volume', (v) => state.study.currentMedia && (state.study.currentMedia.volume = v));
                    this.onConfigChange('playbackRate', (r) => state.study.currentMedia && (state.study.currentMedia.playbackRate = parseFloat(r.toString())));
                    const mainContentWin = (_a = (0, easy_us_1.$el)('#mainContent')) === null || _a === void 0 ? void 0 : _a.contentWindow;
                    if (mainContentWin) {
                        // 弹窗强制用户点击，防止视频无法自动播放
                        easy_us_1.$modal.confirm({
                            content: (0, easy_us_1.h)('div', [
                                '是否开始自动学习当前章节？',
                                (0, easy_us_1.h)('br'),
                                '你也可以选择任意的章节进行点击，脚本会自动学习，并一直往下寻找章节。'
                            ]),
                            cancelButtonText: '我想手动选择章节',
                            confirmButtonText: '开始学习',
                            onConfirm() {
                                return __awaiter(this, void 0, void 0, function* () {
                                    study(StudyLock.getLock());
                                    scrollToJob();
                                });
                            }
                        });
                    }
                    if (this.cfg.showScrollBar) {
                        const bar = (0, easy_us_1.$el)('.dumascroll_area', mainContentWin.document);
                        bar && (bar.style.overflow = 'auto');
                    }
                    if (this.cfg.expandAll) {
                        (0, easy_us_1.$$el)('.s_sectionlist,.s_sectionwrap', mainContentWin.document).forEach((el) => (el.style.display = 'block'));
                    }
                    for (const job of (0, easy_us_1.$$el)('.s_point[itemtype]', mainContentWin.document)) {
                        job.addEventListener('click', (e) => {
                            const lock = StudyLock.getLock();
                            // 如果是用户点击
                            if (e.isTrusted) {
                                if (job.getAttribute('itemtype') === 'exam') {
                                    return easy_us_1.$message.info({
                                        duration: 60,
                                        content: '检测到您手动选择了作业/考试章节，将不会自动跳转，请完成后手动选择其他章节，脚本会自动学习。'
                                    });
                                }
                                else {
                                    easy_us_1.$message.info('检测到章节切换，即将自动学习...');
                                }
                            }
                            setTimeout(() => {
                                study(lock);
                            }, 3000);
                        });
                    }
                    const scrollToJob = () => { var _a; return (_a = (0, easy_us_1.$el)('.s_pointerct', mainContentWin.document)) === null || _a === void 0 ? void 0 : _a.scrollIntoView({ behavior: 'smooth', block: 'center' }); };
                    /** 学习 */
                    const study = (studyLock) => __awaiter(this, void 0, void 0, function* () {
                        var _b;
                        const iframe = (0, easy_us_1.$el)('iframe', mainContentWin.document);
                        const win = iframe === null || iframe === void 0 ? void 0 : iframe.contentWindow;
                        if (win) {
                            const doc = win.document;
                            if (iframe.src.includes('content_video.action') || iframe.src.includes('content_audio.action')) {
                                // 视频
                                background_1.$console.log('视频/音频播放中...');
                                try {
                                    const media = yield (0, study_1.waitForMedia)({ root: doc });
                                    state.study.currentMedia = media;
                                    media.playbackRate = parseFloat(this.cfg.playbackRate.toString());
                                    media.volume = this.cfg.volume;
                                    media.currentTime = 0;
                                    yield new Promise((resolve, reject) => {
                                        try {
                                            // @ts-ignore
                                            win.jwplayer().onComplete(() => __awaiter(this, void 0, void 0, function* () {
                                                background_1.$console.log('视频/音频播放完成。');
                                                yield core_1.$.sleep(3000);
                                                resolve();
                                            }));
                                            media.addEventListener('pause', () => __awaiter(this, void 0, void 0, function* () {
                                                if (!media.ended) {
                                                    yield waitForPopupQuestion(doc);
                                                    yield core_1.$.sleep(1000);
                                                    (0, utils_1.playMedia)(() => media.play());
                                                }
                                            }));
                                            // 开始播放
                                            (0, utils_1.playMedia)(() => media.play());
                                        }
                                        catch (err) {
                                            reject(err);
                                        }
                                    });
                                }
                                catch (err) {
                                    easy_us_1.$message.error(String(err));
                                }
                            }
                            else if (iframe.src.includes('content_doc.action')) {
                                // 文档只需点击就算完成，等待5秒下一个
                                yield core_1.$.sleep(5000);
                            }
                        }
                        else {
                            // 如果为 null 证明跨域
                        }
                        background_1.$console.log(this.cfg.switchPeriod + ' 秒后切换下一章节。');
                        yield core_1.$.sleep(this.cfg.switchPeriod * 1000);
                        if (studyLock.canStudy()) {
                            let nextEl;
                            // 是否处于当前章节之后
                            let isBellowCurrentJob = false;
                            const jobs = (0, easy_us_1.$$el)('.s_point[itemtype]', mainContentWin.document);
                            for (let index = 0; index < jobs.length; index++) {
                                const job = jobs[index];
                                if (job.classList.contains('s_pointerct')) {
                                    isBellowCurrentJob = true;
                                }
                                else if (isBellowCurrentJob) {
                                    if (job.querySelector('.done_icon_show') === null || this.cfg.restudy) {
                                        background_1.$console.log('下一章：', job.title || ((_b = (0, easy_us_1.$el)('.s_pointti', job)) === null || _b === void 0 ? void 0 : _b.title) || '未知');
                                        nextEl = job;
                                        break;
                                    }
                                }
                            }
                            if (nextEl) {
                                nextEl.click();
                                scrollToJob();
                            }
                            else {
                                easy_us_1.$modal.alert({ content: '全部任务已完成' });
                                common_1.CommonProject.scripts.settings.methods.notificationBySetting('全部任务点已完成！', {
                                    duration: 0,
                                    extraTitle: '智慧职教学习脚本'
                                });
                            }
                        }
                    });
                });
            }
        }),
        work: new easy_us_1.Script({
            name: '✍️ 作业考试脚本',
            matches: [['作业考试页面', '/exam']],
            namespace: 'icve.work',
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        '自动答题前请在 “通用-全局设置” 中设置题库配置。',
                        '可以搭配 “通用-在线搜题” 一起使用。',
                        '请手动进入作业考试页面才能使用自动答题。'
                    ]).outerHTML
                }
            },
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    easy_us_1.$message.info('自动答题时请勿切换题目，否则可能导致重复搜题或者脚本卡主。');
                    // 回到第一题
                    const resetToBegin = () => {
                        var _a;
                        (_a = document.querySelectorAll(`.sheet_nums [id*="sheetSeq"]`).item(0)) === null || _a === void 0 ? void 0 : _a.click();
                    };
                    (0, work_1.commonWork)(this, {
                        workerProvider: work,
                        beforeRunning: () => __awaiter(this, void 0, void 0, function* () {
                            resetToBegin();
                            yield core_1.$.sleep(1000);
                        }),
                        onRestart: () => resetToBegin()
                    });
                });
            }
        }),
        workDispatcher: new easy_us_1.Script({
            name: '作业调度脚本',
            matches: [
                ['作业进入页面', '/platformwebapi/student/exam/'],
                ['确认作业页面', '/student/exam/studentExam_studentInfo.action']
            ],
            hideInPanel: true,
            oncomplete() {
                if (/\/platformwebapi\/student\/exam/.test(window.location.href)) {
                    easy_us_1.cors.on('icve-work-start', () => {
                        setTimeout(() => {
                            easy_us_1.$gm.unsafeWindow.openExamInfo();
                        }, 3000);
                    });
                }
                if (/\/student\/exam\/studentExam_studentInfo.action/.test(window.location.href)) {
                    setTimeout(() => {
                        // 确认答题后，OCS会自动执行 ICVE.scripts.work 的 oncomplete 然后开始答题
                        easy_us_1.$gm.unsafeWindow.enterExamPage();
                    }, 3000);
                }
            }
        })
    }
});
function work({ answererWrappers, period, thread, answerSeparators, answerMatchMode }) {
    easy_us_1.$message.info('开始作业');
    common_1.CommonProject.scripts.workResults.methods.init();
    console.log({ answererWrappers, period, thread });
    const titleTransform = (titles) => {
        return titles
            .filter((t) => t === null || t === void 0 ? void 0 : t.innerText)
            .map((t) => {
            var _a, _b;
            if (t) {
                const title = t.cloneNode(true);
                (_a = title.querySelector('[name*="questionIndex"]')) === null || _a === void 0 ? void 0 : _a.remove();
                (_b = title.querySelector('.q_score')) === null || _b === void 0 ? void 0 : _b.remove();
                return title.innerText.trim().replace(/^、/, '') || '';
            }
            return '';
        })
            .join(',');
    };
    const workResults = [];
    let totalQuestionCount = 0;
    let requestedCount = 0;
    let resolvedCount = 0;
    function getType(options) {
        const radio_len = options
            .map((o) => o.querySelector('[type="radio"]'))
            .reduce((a, b) => {
            return a + (b ? 1 : 0);
        }, 0);
        return radio_len > 0
            ? radio_len === 2
                ? 'judgement'
                : 'single'
            : options.some((o) => o.querySelector('[type="checkbox"]'))
                ? 'multiple'
                : options.some((o) => o.querySelector('textarea'))
                    ? 'completion'
                    : options.some((o) => o.querySelector('.fillblank_input input'))
                        ? 'fill-blank'
                        : undefined;
    }
    const worker = new core_1.OCSWorker({
        root: '.q_content',
        elements: {
            title: '.divQuestionTitle, ' +
                // 单行填空题
                '[name="fillblankTitle"]',
            options: '.questionOptions .q_option, .questionOptions.divTextarea, ' +
                // 单行填空题
                '.answerOption'
        },
        thread: thread !== null && thread !== void 0 ? thread : 1,
        answerSeparators: answerSeparators.split(',').map((s) => s.trim()),
        answerMatchMode: answerMatchMode,
        /** 默认搜题方法构造器 */
        answerer: (elements, ctx) => {
            const title = titleTransform(elements.title);
            if (title) {
                return common_1.CommonProject.scripts.apps.methods.searchAnswerInCaches(title, () => __awaiter(this, void 0, void 0, function* () {
                    yield core_1.$.sleep((period !== null && period !== void 0 ? period : 3) * 1000);
                    return (0, core_1.defaultAnswerWrapperHandler)(answererWrappers, {
                        type: getType(ctx.elements.options) || 'unknown',
                        title,
                        options: ctx.elements.options.map((o) => o.innerText).join('\n')
                    });
                }));
            }
            else {
                throw new Error('题目为空，请查看题目是否为空，或者忽略此题');
            }
        },
        work(ctx) {
            return __awaiter(this, void 0, void 0, function* () {
                const options = ctx.elements.options;
                const type = getType(options);
                if (!type) {
                    throw new Error('无法获取题目类型！');
                }
                if (type === 'fill-blank') {
                    const inputs = options
                        .map((o) => Array.from(o.querySelectorAll('.fillblank_input input')))
                        .flat();
                    for (const searchInfo of ctx.searchInfos) {
                        for (const result of searchInfo.results) {
                            const answers = (0, core_1.splitAnswer)(result.answer);
                            if (answers.length === inputs.length) {
                                for (let index = 0; index < inputs.length; index++) {
                                    inputs[index].value = answers[index];
                                }
                                return { finish: true };
                            }
                        }
                    }
                }
                else {
                    const resolver = (0, core_1.defaultQuestionResolve)(ctx)[type];
                    const res = yield resolver(ctx.searchInfos, ctx.elements.options, (type, answer, option) => {
                        var _a, _b;
                        if (type === 'judgement' || type === 'single' || type === 'multiple') {
                            // 这里只用判断多选题是否选中，如果选中就不用再点击了，单选题是 radio，所以不用判断。
                            if (option.querySelector('.checkbox_on') === null) {
                                (_a = (0, easy_us_1.$el)('div', option)) === null || _a === void 0 ? void 0 : _a.click();
                            }
                        }
                        else if (type === 'completion' && answer.trim()) {
                            const text = option.querySelector('textarea');
                            const textIframe = option.querySelector('iframe[id*="ueditor"]');
                            if (text) {
                                text.value = answer;
                            }
                            if (textIframe) {
                                const view = (_b = textIframe.contentWindow) === null || _b === void 0 ? void 0 : _b.document.querySelector('body.view > p');
                                if (view) {
                                    view.innerText = answer;
                                }
                            }
                        }
                    });
                    return res;
                }
                return { finish: false };
            });
        },
        onElementSearched(elements, root) {
            console.log('elements', elements);
        },
        /**
         * 因为校内课的考试和作业都是一题一题做的，不像其他自动答题一样可以获取全部试卷内容。
         * 所以只能根据自定义的状态进行搜索结果的显示。
         */
        onResultsUpdate(currentResult) {
            var _a;
            if (currentResult.resolved) {
                workResults.push(...(0, work_1.simplifyWorkResult)([currentResult], titleTransform));
                common_1.CommonProject.scripts.workResults.methods.setResults(workResults);
                totalQuestionCount++;
                requestedCount++;
                resolvedCount++;
                if ((_a = currentResult.result) === null || _a === void 0 ? void 0 : _a.finish) {
                    common_1.CommonProject.scripts.apps.methods.addQuestionCacheFromWorkResult((0, work_1.simplifyWorkResult)([currentResult], titleTransform));
                }
                common_1.CommonProject.scripts.workResults.methods.updateWorkState({
                    totalQuestionCount,
                    requestedCount,
                    resolvedCount
                });
            }
        }
    });
    const getNextBtn = () => document.querySelector('.paging_next');
    let next = getNextBtn();
    (() => __awaiter(this, void 0, void 0, function* () {
        while (next && worker.isClose === false) {
            yield worker.doWork({ enable_debug: true });
            yield core_1.$.sleep(1000);
            next = getNextBtn();
            if (next.style.display === 'none') {
                break;
            }
            else {
                next === null || next === void 0 ? void 0 : next.click();
                yield core_1.$.sleep(1000);
            }
        }
        easy_us_1.$message.info({ content: '作业/考试完成，请自行检查后保存或提交。', duration: 0 });
        worker.emit('done');
        // 搜索完成后才会同步答案与题目的显示，防止题目错乱
        common_1.CommonProject.scripts.workResults.cfg.questionPositionSyncHandlerType = 'icve';
    }))();
    return worker;
}
/**
 * 等待弹出的答题框，并点击确定
 */
function waitForPopupQuestion(dom) {
    return new Promise((resolve) => {
        const interval = setInterval(() => {
            var _a, _b;
            const el = (0, easy_us_1.$el)('.popup-test', dom);
            if (el) {
                clearInterval(interval);
                const right_answer = ((_a = (0, easy_us_1.$el)('#right_answer', el)) === null || _a === void 0 ? void 0 : _a.value) || 'A';
                for (const answer of right_answer.split('')) {
                    const item = (0, easy_us_1.$el)(`li.test-item-cell[curval="${answer}"]`, el);
                    item === null || item === void 0 ? void 0 : item.click();
                }
                (_b = (0, easy_us_1.$el)('[name="save_btn"]', el)) === null || _b === void 0 ? void 0 : _b.click();
                setTimeout(() => {
                    var _a;
                    (_a = (0, easy_us_1.$el)('[name="continue_btn"]', el)) === null || _a === void 0 ? void 0 : _a.click();
                    resolve();
                }, 3000);
            }
        }, 1000);
    });
}
