"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZHSProject = void 0;
const easy_us_1 = require("easy-us");
const core_1 = require("@ocsjs/core");
const common_1 = require("./common");
const configs_1 = require("../utils/configs");
const work_1 = require("../utils/work");
const utils_1 = require("../utils");
const background_1 = require("./background");
const study_1 = require("../utils/study");
const app_1 = require("../utils/app");
const state = {
    study: {
        /**
         * 学习是否暂停
         */
        stop: false,
        currentMedia: undefined,
        stopInterval: 0,
        stopMessage: undefined
    }
};
/** 工程导出 */
exports.ZHSProject = easy_us_1.Project.create({
    name: '知到智慧树',
    domains: ['zhihuishu.com'],
    scripts: {
        guide: new easy_us_1.Script({
            name: '💡 使用提示',
            matches: [
                ['学习首页', 'https://onlineweb.zhihuishu.com/onlinestuh5'],
                ['首页', 'https://www.zhihuishu.com/']
            ],
            namespace: 'zhs.guide',
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        '请手动进入视频、作业、考试页面，脚本会自动运行。',
                        '兴趣课会自动下一个，所以不提供脚本。'
                    ]).outerHTML
                }
            },
            oncomplete() {
                // 置顶
                common_1.CommonProject.scripts.render.methods.pin(this);
            }
        }),
        'gxk-study': new easy_us_1.Script({
            name: '🖥️ 共享课-学习脚本',
            matches: [['共享课学习页面', 'studyvideoh5.zhihuishu.com']],
            namespace: 'zhs.gxk.study',
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        '章节测试请大家观看完视频后手动打开。',
                        [
                            '请大家仔细打开视频上方的”学前必读“，查看成绩分布。',
                            '如果 “平时成绩-学习习惯成绩” 占比多的话，就需要规律学习。',
                            '每天定时半小时可获得一分习惯分。',
                            '如果不想要习惯分可忽略。'
                        ],
                        '不要最小化浏览器，可能导致脚本暂停。',
                        '运行中请将浏览器缩放调整至适合的大小，避免元素遮挡，无法点击'
                    ]).outerHTML
                },
                /** 学习记录 []  */
                studyRecord: {
                    defaultValue: [],
                    extra: {
                        appConfigSync: false
                    }
                },
                stopTime: {
                    label: '定时停止',
                    tag: 'select',
                    attrs: { title: '到时间后自动暂停脚本' },
                    defaultValue: '0',
                    options: [
                        ['0', '关闭'],
                        ['0.5', '半小时后'],
                        ['1', '一小时后'],
                        ['2', '两小时后']
                    ]
                },
                restudy: configs_1.restudy,
                volume: configs_1.volume,
                definition: configs_1.definition,
                playbackRate: {
                    label: '视频倍速',
                    tag: 'select',
                    defaultValue: 1,
                    options: [
                        ['1', '1 x'],
                        ['1.25', '1.25 x'],
                        ['1.5', '1.5 x']
                    ]
                }
            },
            methods() {
                return {
                    /**
                     * 增加学习时间
                     * @param courseName 课程名
                     * @param val 增加的时间
                     */
                    increaseStudyTime: (courseName, val) => {
                        const records = this.cfg.studyRecord;
                        // 查找是否存在今天的记录
                        const record = records.find((r) => new Date(r.date).toLocaleDateString() === new Date().toLocaleDateString());
                        let courses = [];
                        if (record) {
                            courses = record.courses;
                        }
                        else {
                            records.push({ date: Date.now(), courses: courses });
                        }
                        // 查找是否存在课程记录
                        const course = courses.find((c) => c.name === courseName);
                        if (course) {
                            // 存在则累加时间
                            course.time = course.time + val;
                            // 历史遗留问题，之前的倍速没有转换为数字，导致可能显示为字符串
                            if (typeof course.time === 'string') {
                                course.time = parseFloat(course.time);
                            }
                        }
                        else {
                            // 不存在则新建
                            courses.push({ name: courseName, time: 0 });
                        }
                        this.cfg.studyRecord = records;
                    }
                };
            },
            onrender({ panel }) {
                panel.body.replaceChildren((0, easy_us_1.h)('hr'), easy_us_1.$ui.button('⏰检测是否需要规律学习', {}, (btn) => {
                    btn.style.marginRight = '12px';
                    btn.onclick = () => {
                        var _a;
                        (_a = (0, easy_us_1.$el)('.iconbaizhoumoshi-xueqianbidu')) === null || _a === void 0 ? void 0 : _a.click();
                        setTimeout(() => {
                            var _a;
                            const pmd = (0, easy_us_1.$el)('.preschool-Mustread-div');
                            if (pmd) {
                                const div = document.querySelector('.dialog-read');
                                if (div) {
                                    div.style.display = 'block';
                                }
                                const num = parseInt(((_a = pmd.innerText.match(/学习习惯成绩（(\d+)分）/)) === null || _a === void 0 ? void 0 : _a[1]) || '0');
                                easy_us_1.$modal.alert({
                                    content: `当前课程习惯分占比为${num}分，` +
                                        (num
                                            ? `需要规律学习${num}天, 每天定时观看半小时即可。（如果不想拿习惯分可以忽略）`
                                            : '可一直观看学习，无需定时停止。')
                                });
                            }
                            else {
                                easy_us_1.$modal.alert({ content: '检测失败，请确认在视频学习页面使用此按钮。' });
                            }
                        }, 100);
                    };
                }), easy_us_1.$ui.button('📘查看学习记录', {}, (btn) => {
                    btn.onclick = () => {
                        easy_us_1.$modal.alert({
                            title: '学习记录',
                            content: easy_us_1.$ui.notes(this.cfg.studyRecord.map((r) => {
                                const date = new Date(r.date);
                                return [
                                    `${date.getFullYear()}-${(date.getMonth() + 1).toString().padStart(2, '0')}-${date
                                        .getDate()
                                        .toString()
                                        .padStart(2, '0')}`,
                                    easy_us_1.$ui.notes(r.courses.map((course) => `${course.name} - ${optimizeSecond(course.time)}`))
                                ];
                            }))
                        });
                    };
                }));
            },
            onactive() {
                // 重置时间
                this.cfg.stopTime = '0';
                if (this.cfg.playbackRate) {
                    // 转换为数字
                    this.cfg.playbackRate = parseFloat(this.cfg.playbackRate.toString());
                }
            },
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    // 置顶当前脚本
                    common_1.CommonProject.scripts.render.methods.pin(this);
                    // 10秒后还没加载出来，则结束
                    setTimeout(() => {
                        if ((0, easy_us_1.$$el)('.clearfix.video').length === 0) {
                            finish();
                        }
                    }, 10 * 1000);
                    const waitForVideoList = () => {
                        return new Promise((resolve, reject) => {
                            if ((0, easy_us_1.$$el)('.clearfix.video').length > 1) {
                                resolve();
                            }
                            else {
                                setTimeout(() => {
                                    resolve(waitForVideoList());
                                }, 1000);
                            }
                        });
                    };
                    yield waitForVideoList();
                    // 检查是否为软件环境
                    const remotePage = yield core_1.RemotePlaywright.getCurrentPage();
                    // 检查是否为软件环境
                    if (!remotePage) {
                        return app_1.$playwright.showError();
                    }
                    // 监听定时停止
                    this.onConfigChange('stopTime', (stopTime) => {
                        if (stopTime === '0') {
                            easy_us_1.$message.info({ content: '定时停止已关闭' });
                        }
                        else {
                            autoStop(stopTime);
                        }
                    });
                    // 监听音量
                    this.onConfigChange('volume', (curr) => {
                        state.study.currentMedia && (state.study.currentMedia.volume = curr);
                    });
                    // 监听速度
                    this.onConfigChange('playbackRate', (curr) => {
                        if (typeof curr === 'string') {
                            this.cfg.playbackRate = parseFloat(curr);
                        }
                        switchPlaybackRate(remotePage, this.cfg.playbackRate);
                    });
                    // 监听清晰度
                    this.onConfigChange('definition', (curr) => {
                        switchLine(remotePage, curr);
                    });
                    const hideDialog = () => {
                        /** 隐藏通知弹窗 */
                        (0, easy_us_1.$$el)('.el-dialog__wrapper').forEach((dialog) => {
                            dialog.remove();
                        });
                    };
                    /** 关闭测验弹窗 */
                    const closeTestDialog = () => __awaiter(this, void 0, void 0, function* () {
                        var _a, _b;
                        const tip = (0, easy_us_1.$el)('[role="dialog"][aria-label="提示"]');
                        if ((_b = (_a = tip === null || tip === void 0 ? void 0 : tip.querySelector('.el-message-box__message')) === null || _a === void 0 ? void 0 : _a.textContent) === null || _b === void 0 ? void 0 : _b.includes('未做答的弹题不能关闭')) {
                            const close = tip.querySelector('[aria-label="Close"]');
                            if (close) {
                                yield remotePage.click('[role="dialog"][aria-label="提示"] [aria-label="Close"]');
                                yield easy_us_1.$.sleep(1000);
                            }
                        }
                        const items = (0, easy_us_1.$$el)('#playTopic-dialog .topic .el-pager .number');
                        if (items.length) {
                            for (const item of items) {
                                if (item.classList.contains('active') === false) {
                                    item.click();
                                    yield easy_us_1.$.sleep(500);
                                }
                                const options = (0, easy_us_1.$$el)('#playTopic-dialog ul .topic-item');
                                if (options.length !== 0) {
                                    yield waitForCaptcha();
                                    // 最小化脚本窗口
                                    common_1.CommonProject.scripts.render.methods.moveToEdge();
                                    // 随机选
                                    const random = Math.floor(Math.random() * options.length);
                                    yield easy_us_1.$.sleep(1000);
                                    // nth-child 从1开始
                                    yield remotePage.click(`#playTopic-dialog .topic .radio ul > li:nth-child(${random + 1})`);
                                    yield easy_us_1.$.sleep(1000);
                                }
                            }
                            yield easy_us_1.$.sleep(1000);
                            // 关闭弹窗
                            yield remotePage.click('#playTopic-dialog .dialog-footer .btn');
                        }
                        /**
                         * 每过三秒递归检测是否有弹窗
                         */
                        yield easy_us_1.$.sleep(3000);
                        yield closeTestDialog();
                    });
                    const finish = () => {
                        easy_us_1.$modal.alert({
                            content: '检测到当前视频全部播放完毕，如果还有未完成的视频请刷新重试，或者打开复习模式。'
                        });
                    };
                    // 循环记录学习时间
                    const recordStudyTimeLoop = () => {
                        var _a;
                        this.methods.increaseStudyTime(((_a = (0, easy_us_1.$el)('.source-name')) === null || _a === void 0 ? void 0 : _a.innerText) || '无名称', this.cfg.playbackRate);
                        setTimeout(recordStudyTimeLoop, 1000);
                    };
                    // 查找任务
                    const findVideoItem = (opts) => {
                        let videoItems = Array.from(document.querySelectorAll('.clearfix.video'));
                        // 如果不是复习模式，则排除掉已经完成的任务
                        if (!this.cfg.restudy) {
                            videoItems = videoItems.filter((el) => el.querySelector('.time_icofinish') === null);
                        }
                        for (let i = 0; i < videoItems.length; i++) {
                            const item = videoItems[i];
                            if (item.classList.contains('current_play')) {
                                return videoItems[i + (opts.next ? 1 : 0)];
                            }
                        }
                        return videoItems[0];
                    };
                    // 检测是否需要学前必读
                    closeDialogRead();
                    // 循环记录学习时间
                    recordStudyTimeLoop();
                    // 自动隐藏弹窗
                    hideDialog();
                    // 自动暂停
                    autoStop(this.cfg.stopTime);
                    // 自动过弹窗测验
                    closeTestDialog();
                    setInterval(() => __awaiter(this, void 0, void 0, function* () {
                        // 删除遮罩层
                        (0, easy_us_1.$$el)('.v-modal,.mask').forEach((modal) => {
                            modal.remove();
                        });
                        // 定时显示进度条，防止消失
                        fixProcessBar();
                    }), 3000);
                    easy_us_1.$message.info({ content: '3秒后开始学习', duration: 3 });
                    const study = (opts) => __awaiter(this, void 0, void 0, function* () {
                        if (state.study.stop === false) {
                            const item = findVideoItem(opts);
                            if (item) {
                                yield easy_us_1.$.sleep(3000);
                                // 最小化脚本窗口
                                common_1.CommonProject.scripts.render.methods.moveToEdge();
                                // 点击侧边栏任务
                                yield remotePage.click(item);
                                watch(remotePage, { volume: this.cfg.volume, playbackRate: this.cfg.playbackRate, definition: this.cfg.definition }, ({ next }) => study({ next }));
                            }
                            else {
                                finish();
                            }
                        }
                        else {
                            easy_us_1.$message.warn({
                                content: '检测到当前视频全部播放完毕，如果还有未完成的视频请刷新重试，或者打开复习模式。'
                            });
                            common_1.CommonProject.scripts.settings.methods.notificationBySetting('检测到当前视频全部播放完毕，如果还有未完成的视频请刷新重试，或者打开复习模式。', { duration: 0, extraTitle: '知道智慧树学习脚本' });
                        }
                    });
                    // 当页面初始化时无需切换下一个视频，直接播放当前的。
                    study({ next: false });
                });
            }
        }),
        'gxk-work': new easy_us_1.Script({
            name: '✍️ 共享课-作业考试脚本',
            matches: [
                ['共享课作业页面', 'zhihuishu.com/stuExamWeb.html#/webExamList/dohomework'],
                ['共享课考试页面', 'zhihuishu.com/stuExamWeb.html#/webExamList/doexamination'],
                ['作业考试列表', 'zhihuishu.com/stuExamWeb.html#/webExamList\\?']
            ],
            namespace: 'zhs.gxk.work',
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes([
                        '📢 如果未开始答题，请尝试刷新页面。',
                        '自动答题前请在 “通用-全局设置” 中设置题库配置。',
                        '可以搭配 “通用-在线搜题” 一起使用。'
                    ]).outerHTML
                }
            },
            methods() {
                function getWorkInfo(remotePage) {
                    return __awaiter(this, void 0, void 0, function* () {
                        const isExam = location.href.includes('doexamination');
                        let url = '';
                        if (isExam) {
                            url = '/taurusExam/gateway/t/v1/student/doExam';
                        }
                        else {
                            url = '/studentExam/gateway/t/v1/student/doHomework';
                        }
                        return JSON.parse((yield remotePage.waitForResponse(url)).body);
                    });
                }
                return {
                    getWorkInfo: getWorkInfo,
                    work: () => __awaiter(this, void 0, void 0, function* () {
                        // 检查是否为软件环境
                        const remotePage = yield core_1.RemotePlaywright.getCurrentPage();
                        // 检查是否为软件环境
                        if (!remotePage) {
                            return app_1.$playwright.showError();
                        }
                        // 等待试卷加载
                        const isExam = location.href.includes('doexamination');
                        const isWork = location.href.includes('dohomework');
                        if (isExam || isWork) {
                            const workInfo = yield getWorkInfo(remotePage);
                            setTimeout(() => {
                                easy_us_1.$message.info({ content: `开始${isExam ? '考试' : '作业'}` });
                                (0, work_1.commonWork)(this, {
                                    workerProvider: (opts) => gxkWorkAndExam(workInfo, opts)
                                });
                            }, 1000);
                        }
                        else {
                            easy_us_1.$message.info({ content: '📢 请手动进入作业/考试，如果未开始答题，请尝试刷新页面。', duration: 0 });
                            common_1.CommonProject.scripts.render.methods.pin(this);
                        }
                    })
                };
            },
            onactive() {
                return __awaiter(this, void 0, void 0, function* () {
                    this.methods.work();
                    /**
                     * 当页面从作业考试列表跳转到作业考试页面时，触发的是onhistorychange事件，而不是oncomplete事件。
                     */
                    this.on('historychange', () => {
                        this.methods.work();
                    });
                });
            }
        }),
        'xnk-study': new easy_us_1.Script({
            name: '🖥️ 校内课-学习脚本',
            matches: [['校内课学习页面', 'zhihuishu.com/aidedteaching/sourceLearning']],
            namespace: 'zhs.xnk.study',
            configs: {
                notes: {
                    defaultValue: easy_us_1.$ui.notes(['章节测试请大家观看完视频后手动打开。', '此课程不能使用倍速。']).outerHTML
                },
                restudy: configs_1.restudy,
                volume: configs_1.volume
            },
            oncomplete() {
                // 置顶当前脚本
                common_1.CommonProject.scripts.render.methods.pin(this);
                const finish = () => {
                    easy_us_1.$modal.alert({
                        content: '检测到当前视频全部播放完毕，如果还有未完成的视频请刷新重试，或者打开复习模式。'
                    });
                    common_1.CommonProject.scripts.settings.methods.notificationBySetting('检测到当前视频全部播放完毕，如果还有未完成的视频请刷新重试，或者打开复习模式。', { duration: 0, extraTitle: '知道智慧树学习脚本' });
                };
                // 监听音量
                this.onConfigChange('volume', (curr) => {
                    state.study.currentMedia && (state.study.currentMedia.volume = curr);
                });
                const nextElement = () => {
                    const list = document.querySelectorAll('.file-item');
                    let passActive = false;
                    for (let index = 0; index < list.length; index++) {
                        const item = list[index];
                        const finish = !!item.querySelector('.icon-finish');
                        // 判断是否需要学习
                        const needsStudy = !finish || (finish && this.cfg.restudy);
                        if (item.classList.contains('active')) {
                            if (needsStudy) {
                                return item;
                            }
                            else {
                                passActive = true;
                            }
                        }
                        if (passActive && needsStudy) {
                            return item;
                        }
                    }
                };
                const interval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    /** 查找任务 */
                    const next = nextElement();
                    if (next) {
                        clearInterval(interval);
                        if (document.querySelector('#mediaPlayer')) {
                            watchXnk({ volume: this.cfg.volume }, () => {
                                setTimeout(() => {
                                    /** 下一章 */
                                    const next = nextElement();
                                    if (next)
                                        next.click();
                                }, 3000);
                            });
                        }
                        else {
                            setTimeout(() => {
                                background_1.$console.log('不是视频任务，即将切换下一章。');
                                /** 下一章 */
                                const next = nextElement();
                                if (next)
                                    next.click();
                            }, 3000);
                        }
                    }
                }), 1000);
                setTimeout(() => {
                    if (!nextElement()) {
                        finish();
                        clearInterval(interval);
                    }
                }, 10 * 1000);
            }
        }),
        'xnk-work': new easy_us_1.Script({
            name: '✍️ 校内课-作业考试脚本',
            matches: [
                ['校内课作业页面', 'zhihuishu.com/atHomeworkExam/stu/homeworkQ/exerciseList'],
                ['校内课考试页面', 'zhihuishu.com/atHomeworkExam/stu/examQ/examexercise']
            ],
            namespace: 'zhs.xnk.work',
            configs: { notes: configs_1.workNotes },
            oncomplete() {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, work_1.commonWork)(this, {
                        workerProvider: xnkWork
                    });
                });
            }
        })
    }
});
/**
 * 观看视频
 * @param setting
 * @returns
 */
function watch(remotePage, options, onended) {
    return __awaiter(this, void 0, void 0, function* () {
        // 部分用户视频加载很慢，这里等待一下
        yield (0, study_1.waitForMedia)();
        const set = () => __awaiter(this, void 0, void 0, function* () {
            // 设置清晰度
            yield switchLine(remotePage, options.definition);
            yield easy_us_1.$.sleep(1000);
            // 设置播放速度
            yield switchPlaybackRate(remotePage, options.playbackRate);
            // 上面操作会导致元素刷新，这里重新获取视频
            const video = yield (0, study_1.waitForMedia)();
            state.study.currentMedia = video;
            if (video) {
                // 如果已经播放完了，则重置视频进度
                video.currentTime = 1;
                // 音量
                video.volume = options.volume;
            }
            return video;
        });
        const video = yield set();
        const videoCheckInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
            // 如果视频元素无法访问，证明已经切换了视频
            if ((video === null || video === void 0 ? void 0 : video.isConnected) === false) {
                clearInterval(videoCheckInterval);
                easy_us_1.$message.info({ content: '检测到视频切换中...' });
                /**
                 * 元素无法访问证明用户切换视频了
                 * 所以不往下播放视频，而是重新播放用户当前选中的视频
                 */
                onended({ next: false });
            }
        }), 3000);
        (0, utils_1.playMedia)(() => video === null || video === void 0 ? void 0 : video.play());
        video.onpause = () => __awaiter(this, void 0, void 0, function* () {
            if (!(video === null || video === void 0 ? void 0 : video.ended) && state.study.stop === false) {
                yield waitForCaptcha();
                yield easy_us_1.$.sleep(1000);
                video === null || video === void 0 ? void 0 : video.play();
            }
        });
        video.onended = () => {
            clearInterval(videoCheckInterval);
            // 正常切换下一个视频
            onended({ next: true });
        };
    });
}
/**
 * 观看校内课
 */
function watchXnk(options, onended) {
    return __awaiter(this, void 0, void 0, function* () {
        // 部分用户视频加载很慢，这里等待一下
        const media = yield (0, study_1.waitForMedia)();
        media.volume = options.volume;
        media.currentTime = 1;
        state.study.currentMedia = media;
        (0, utils_1.playMedia)(() => media === null || media === void 0 ? void 0 : media.play());
        media.onpause = () => __awaiter(this, void 0, void 0, function* () {
            if (!(media === null || media === void 0 ? void 0 : media.ended)) {
                yield easy_us_1.$.sleep(1000);
                media === null || media === void 0 ? void 0 : media.play();
            }
        });
        media.onended = () => {
            // 正常切换下一个视频
            onended();
        };
    });
}
/**
 * 切换视频清晰度
 * @param definition 清晰度的类名
 */
function switchLine(remotePage, definition = 'line1bq') {
    return __awaiter(this, void 0, void 0, function* () {
        const controlsBar = (0, easy_us_1.$el)('.controlsBar');
        const dl = (0, easy_us_1.$el)('.definiLines');
        if (controlsBar && dl) {
            controlsBar.style.display = 'block';
            dl.style.display = 'block';
            yield remotePage.click(`.definiLines .${definition}`);
        }
    });
}
/**
 * 切换视频清晰度
 * @param playbackRate 播放速度
 */
function switchPlaybackRate(remotePage, playbackRate) {
    return __awaiter(this, void 0, void 0, function* () {
        const controlsBar = (0, easy_us_1.$el)('.controlsBar');
        const sl = (0, easy_us_1.$el)('.speedList');
        if (controlsBar && sl) {
            controlsBar.style.display = 'block';
            sl.style.display = 'block';
            yield remotePage.click(`.speedList [rate="${playbackRate === 1 ? '1.0' : playbackRate}"]`);
        }
    });
}
/**
 * 检测是否有验证码，并等待验证
 */
function checkForCaptcha(update) {
    let modal;
    let notified = false;
    return setInterval(() => {
        if ((0, easy_us_1.$el)('.yidun_popup')) {
            update(true);
            // 如果弹窗不存在，则显示
            if (modal === undefined) {
                modal = easy_us_1.$modal.alert({ content: '当前检测到验证码，请输入后方可继续运行。' });
            }
            // 如果没有通知过，则通知
            if (!notified) {
                notified = true;
                common_1.CommonProject.scripts.settings.methods.notificationBySetting('智慧树脚本：当前检测到验证码，请输入后方可继续运行。', { duration: 0 });
            }
        }
        else {
            if (modal) {
                update(false);
                // 关闭弹窗
                modal.remove();
                modal = undefined;
            }
        }
    }, 1000);
}
function waitForCaptcha() {
    const popup = getPopupCaptcha();
    if (popup) {
        easy_us_1.$message.warn({ content: '当前检测到验证码，请输入后方可继续运行。' });
        common_1.CommonProject.scripts.settings.methods.notificationBySetting('智慧树脚本：当前检测到验证码，请输入后方可继续运行。', { duration: 0 });
        return new Promise((resolve, reject) => {
            const interval = setInterval(() => {
                const popup = getPopupCaptcha();
                if (popup === null) {
                    clearInterval(interval);
                    resolve();
                }
            }, 1000);
        });
    }
}
function getPopupCaptcha() {
    return document.querySelector('.yidun_popup');
}
/**
 * 共享课的作业和考试
 */
function gxkWorkAndExam(workInfo, { answererWrappers, period, thread, stopSecondWhenFinish, redundanceWordsText, answerSeparators, answerMatchMode }) {
    var _a, _b, _c;
    common_1.CommonProject.scripts.workResults.methods.init({
        questionPositionSyncHandlerType: 'zhs-gxk'
    });
    /**
     * workExamParts 是个列表
     * 里面包括一个题目类型的列表，第一个是单选，第二个是多选，第三个是判断
     * 所以这里直接扁平化数组方便处理
     */
    const allExamParts = ((_c = (((_b = (_a = workInfo === null || workInfo === void 0 ? void 0 : workInfo.rt) === null || _a === void 0 ? void 0 : _a.examBase) === null || _b === void 0 ? void 0 : _b.workExamParts) || [])) === null || _c === void 0 ? void 0 : _c.map((p) => p.questionDtos).flat()) || [];
    const titleTransform = (_, index) => {
        var _a;
        const div = (0, easy_us_1.h)('div');
        div.innerHTML = ((_a = allExamParts[index]) === null || _a === void 0 ? void 0 : _a.name) || '题目读取失败';
        return (0, work_1.removeRedundantWords)((0, work_1.optimizationElementWithImage)(div).innerText || '', redundanceWordsText.split('\n'));
    };
    let request_index = 0;
    /** 新建答题器 */
    const worker = new core_1.OCSWorker({
        root: '.examPaper_subject',
        elements: {
            /**
             * .subject_describe > div: 选择题题目
             * .smallStem_describe > div:nth-child(2): 阅读理解小题题目
             */
            title: '.subject_describe > div,.smallStem_describe > div:nth-child(2)',
            // 选项中图片识别
            options: (root) => (0, easy_us_1.$$el)('.subject_node .nodeLab', root).map((t) => {
                for (const img of Array.from(t.querySelectorAll('.node_detail img'))) {
                    // zhs选项中如果已显示的图片则不存在 data-src，如果未显示则存在 data-src
                    if (img.dataset.src) {
                        img.src = img.dataset.src;
                    }
                    // 不使用 optimizationElementWithImage 是因为zhs的选项按钮也是一个图片
                    (0, work_1.createUnVisibleTextOfImage)(img);
                }
                return t;
            })
        },
        thread: thread !== null && thread !== void 0 ? thread : 1,
        answerSeparators: answerSeparators.split(',').map((s) => s.trim()),
        answerMatchMode: answerMatchMode,
        /** 默认搜题方法构造器 */
        answerer: (elements, ctx) => {
            const title = titleTransform(undefined, request_index++);
            if (title) {
                return common_1.CommonProject.scripts.apps.methods.searchAnswerInCaches(title, () => __awaiter(this, void 0, void 0, function* () {
                    yield easy_us_1.$.sleep((period !== null && period !== void 0 ? period : 3) * 1000);
                    return (0, core_1.defaultAnswerWrapperHandler)(answererWrappers, {
                        type: ctx.type || 'unknown',
                        title,
                        options: ctx.elements.options.map((o) => o.innerText).join('\n')
                    });
                }));
            }
            else {
                throw new Error('题目为空，请查看题目是否为空，或者忽略此题');
            }
        },
        work: {
            type(ctx) {
                var _a, _b, _c, _d;
                const type = (_d = (_c = (_b = (_a = ctx.elements.title[0].parentElement) === null || _a === void 0 ? void 0 : _a.parentElement) === null || _b === void 0 ? void 0 : _b.querySelector('.subject_type')) === null || _c === void 0 ? void 0 : _c.textContent) === null || _d === void 0 ? void 0 : _d.trim();
                if (type === null || type === void 0 ? void 0 : type.includes('单选题')) {
                    return 'single';
                }
                else if (type === null || type === void 0 ? void 0 : type.includes('多选题')) {
                    return 'multiple';
                }
                else if (type === null || type === void 0 ? void 0 : type.includes('判断题')) {
                    return 'judgement';
                }
                else if (type === null || type === void 0 ? void 0 : type.includes('填空题')) {
                    return 'completion';
                }
                else {
                    return undefined;
                }
            },
            /** 自定义处理器 */
            handler(type, answer, option) {
                var _a;
                return __awaiter(this, void 0, void 0, function* () {
                    if (type === 'judgement' || type === 'single' || type === 'multiple') {
                        if (!((_a = option.querySelector('input')) === null || _a === void 0 ? void 0 : _a.checked)) {
                            option.click();
                            yield easy_us_1.$.sleep(200);
                        }
                    }
                    else if (type === 'completion' && answer.trim()) {
                        const text = option.querySelector('textarea');
                        if (text) {
                            text.value = answer;
                            yield easy_us_1.$.sleep(200);
                        }
                    }
                });
            }
        },
        /** 完成答题后 */
        onResultsUpdate(curr, index, res) {
            var _a, _b;
            common_1.CommonProject.scripts.workResults.methods.setResults((0, work_1.simplifyWorkResult)(res, titleTransform));
            if ((_a = curr.result) === null || _a === void 0 ? void 0 : _a.finish) {
                const title = (_b = allExamParts[index]) === null || _b === void 0 ? void 0 : _b.name;
                if (title) {
                    common_1.CommonProject.scripts.apps.methods.addQuestionCacheFromWorkResult((0, work_1.simplifyWorkResult)([curr], (_, __) => title));
                }
            }
            common_1.CommonProject.scripts.workResults.methods.updateWorkStateByResults(res);
        }
    });
    checkForCaptcha((hasCaptcha) => {
        if (hasCaptcha) {
            worker.emit('stop');
        }
        else {
            worker.emit('continuate');
        }
    });
    worker
        .doWork()
        .then((res) => __awaiter(this, void 0, void 0, function* () {
        var _d;
        // 如果被强制关闭，则不进行保存操作
        if (worker.isClose === true) {
            return;
        }
        easy_us_1.$message.success({ content: `答题完成，将等待 ${stopSecondWhenFinish} 秒后进行保存或提交。` });
        yield easy_us_1.$.sleep(stopSecondWhenFinish * 1000);
        // @ts-ignore
        if (worker.isClose === true) {
            return;
        }
        /**
         * 保存题目，不在选择答案后保存的原因是，如果答题线程大于3会导致题目错乱，因为 resolverIndex 并不是顺序递增的
         */
        for (let index = 0; index < worker.totalQuestionCount; index++) {
            // @ts-ignore
            if (worker.isClose === true) {
                return;
            }
            const modal = easy_us_1.$modal.alert({
                content: '正在保存题目中（必须保存，否则填写的答案无效），<br>请勿操作...',
                confirmButton: null
            });
            yield waitForCaptcha();
            yield easy_us_1.$.sleep(2000);
            // 跳转到该题目，防止用户在保存时切换题目
            (_d = document.querySelectorAll('.answerCard_list ul li').item(index)) === null || _d === void 0 ? void 0 : _d.click();
            yield easy_us_1.$.sleep(200);
            // 下一页
            const next = (0, easy_us_1.$el)('div.examPaper_box > div.switch-btn-box > button:nth-child(2)');
            if (next) {
                next.click();
            }
            else {
                background_1.$console.error('未找到下一页按钮。');
            }
            modal === null || modal === void 0 ? void 0 : modal.remove();
        }
        easy_us_1.$message.info({ content: '作业/考试完成，请自行检查后保存或提交。', duration: 0 });
        worker.emit('done');
    }))
        .catch((err) => {
        easy_us_1.$message.error({ content: '答题程序发生错误 : ' + err.message, duration: 0 });
    });
    return worker;
}
/**
 * 校内学分课的作业
 */
function xnkWork({ answererWrappers, period, thread, answerSeparators, answerMatchMode }) {
    easy_us_1.$message.info({ content: '开始作业' });
    common_1.CommonProject.scripts.workResults.methods.init();
    const titleTransform = (titles) => {
        return titles
            .filter((t) => t === null || t === void 0 ? void 0 : t.innerText)
            .map((t) => (t ? (0, work_1.optimizationElementWithImage)(t).innerText : ''))
            .join(',');
    };
    const workResults = [];
    let totalQuestionCount = 0;
    let requestedCount = 0;
    let resolvedCount = 0;
    const worker = new core_1.OCSWorker({
        root: '.questionBox',
        elements: {
            title: '.questionContent',
            options: '.optionUl label',
            questionTit: '.questionTit'
        },
        thread: thread !== null && thread !== void 0 ? thread : 1,
        answerSeparators: answerSeparators.split(',').map((s) => s.trim()),
        answerMatchMode: answerMatchMode,
        /** 默认搜题方法构造器 */
        answerer: (elements, ctx) => {
            const title = titleTransform(elements.title);
            if (title) {
                return common_1.CommonProject.scripts.apps.methods.searchAnswerInCaches(title, () => __awaiter(this, void 0, void 0, function* () {
                    yield easy_us_1.$.sleep((period !== null && period !== void 0 ? period : 3) * 1000);
                    return (0, core_1.defaultAnswerWrapperHandler)(answererWrappers, {
                        type: ctx.type || 'unknown',
                        title,
                        options: ctx.elements.options.map((o) => o.innerText).join('\n')
                    });
                }));
            }
            else {
                throw new Error('题目为空，请查看题目是否为空，或者忽略此题');
            }
        },
        work: {
            /** 自定义处理器 */
            handler(type, answer, option, ctx) {
                var _a;
                return __awaiter(this, void 0, void 0, function* () {
                    if (type === 'judgement' || type === 'single' || type === 'multiple') {
                        if (((_a = option.querySelector('input')) === null || _a === void 0 ? void 0 : _a.checked) === false) {
                            option.click();
                            yield easy_us_1.$.sleep(200);
                        }
                    }
                    else if (type === 'completion' && answer.trim()) {
                        const text = option.querySelector('textarea');
                        if (text) {
                            text.value = answer;
                            yield easy_us_1.$.sleep(200);
                        }
                    }
                });
            }
        },
        /**
         * 因为校内课的考试和作业都是一题一题做的，不像其他自动答题一样可以获取全部试卷内容。
         * 所以只能根据自定义的状态进行搜索结果的显示。
         */
        onResultsUpdate(current, _, res) {
            var _a;
            if (current.result) {
                workResults.push(...(0, work_1.simplifyWorkResult)([current], titleTransform));
                common_1.CommonProject.scripts.workResults.methods.setResults(workResults);
                totalQuestionCount++;
                requestedCount++;
                resolvedCount++;
            }
            if ((_a = current.result) === null || _a === void 0 ? void 0 : _a.finish) {
                common_1.CommonProject.scripts.apps.methods.addQuestionCacheFromWorkResult((0, work_1.simplifyWorkResult)([current], titleTransform));
            }
            common_1.CommonProject.scripts.workResults.methods.updateWorkState({
                totalQuestionCount,
                requestedCount,
                resolvedCount
            });
        }
    });
    const getBtn = () => document.querySelector('span.Topicswitchingbtn:nth-child(2)');
    let next = getBtn();
    (() => __awaiter(this, void 0, void 0, function* () {
        while (next && worker.isClose === false) {
            yield worker.doWork();
            yield easy_us_1.$.sleep(1000);
            next = getBtn();
            next === null || next === void 0 ? void 0 : next.click();
            yield easy_us_1.$.sleep(1000);
        }
        easy_us_1.$message.info({ content: '作业/考试完成，请自行检查后保存或提交。', duration: 0 });
        worker.emit('done');
        common_1.CommonProject.scripts.workResults.cfg.questionPositionSyncHandlerType = 'zhs-xnk';
    }))();
    return worker;
}
/**
 * 将秒数转换为小时或分钟
 * @param second 秒
 */
function optimizeSecond(second) {
    if (second > 3600) {
        return `${Math.floor(second / 3600)}小时${Math.floor((second % 3600) / 60)}分钟`;
    }
    else if (second > 60) {
        return `${Math.floor(second / 60)}分钟${second % 60}秒`;
    }
    else {
        return `${second}秒`;
    }
}
function autoStop(stopTime) {
    var _a, _b;
    clearInterval(state.study.stopInterval);
    (_a = state.study.stopMessage) === null || _a === void 0 ? void 0 : _a.remove();
    if (stopTime !== '0') {
        let stopCount = parseFloat(stopTime) * 60 * 60;
        state.study.stopInterval = setInterval(() => {
            var _a;
            if (stopCount > 0) {
                // 如果有弹窗验证码则暂停自动停止的计时
                if (getPopupCaptcha() === null) {
                    stopCount--;
                }
            }
            else {
                clearInterval(state.study.stopInterval);
                state.study.stop = true;
                (_a = (0, easy_us_1.$el)('video')) === null || _a === void 0 ? void 0 : _a.pause();
                easy_us_1.$modal.alert({ content: '脚本暂停，已获得今日平时分，如需继续观看，请刷新页面。' });
            }
        }, 1000);
        const val = ((_b = exports.ZHSProject.scripts['gxk-study'].configs.stopTime.options.find((t) => t[0] === stopTime)) === null || _b === void 0 ? void 0 : _b[0]) || '0';
        const date = new Date();
        date.setMinutes(date.getMinutes() + parseFloat(val) * 60);
        state.study.stopMessage = easy_us_1.$message.info({
            duration: 0,
            content: `在 ${date.toLocaleTimeString()} 脚本将自动暂停`
        });
    }
}
/** 固定视频进度 */
function fixProcessBar() {
    const bar = document.querySelector('.controlsBar');
    if (bar) {
        bar.style.display = 'block';
    }
}
function closeDialogRead() {
    const div = document.querySelector('.dialog-read');
    if (div) {
        div.style.display = 'none';
    }
}
