"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.enableCopy = exports.playMedia = exports.createRangeTooltip = exports.workPreCheckMessage = void 0;
const easy_us_1 = require("easy-us");
const background_1 = require("../projects/background");
/** 创建答题预处理信息 */
function workPreCheckMessage(options) {
    const { onrun, onNoAnswererWrappers, onclose } = options, opts = __rest(options, ["onrun", "onNoAnswererWrappers", "onclose"]);
    if (opts.answererWrappers.length === 0) {
        onNoAnswererWrappers === null || onNoAnswererWrappers === void 0 ? void 0 : onNoAnswererWrappers(opts);
        return easy_us_1.$message.warn({
            content: '检测到题库配置为空，无法自动答题，请前往 通用-全局设置 页面进行配置。',
            duration: 0
        });
    }
    else {
        return easy_us_1.$message.info({
            duration: 5,
            content: (0, easy_us_1.h)('span', [
                '5秒后自动答题，',
                easy_us_1.$ui.preventText({
                    name: '点击取消',
                    delay: 5,
                    ondefault: (span) => {
                        onrun(opts);
                    },
                    onprevent(span) {
                        const closedMessage = easy_us_1.$message.warn({
                            content: '已关闭此次的自动答题，请手动开启或者忽略此警告。',
                            duration: 0
                        });
                        if (closedMessage) {
                            onclose === null || onclose === void 0 ? void 0 : onclose(opts, closedMessage);
                        }
                    }
                })
            ])
        });
    }
}
exports.workPreCheckMessage = workPreCheckMessage;
/**
 * 创造范围选择器的提示
 */
function createRangeTooltip(input, defaultValue, transform) {
    input.addEventListener('change', () => {
        input.setAttribute('data-title', transform(input.value || input.getAttribute('value') || defaultValue));
    });
    input.setAttribute('data-title', transform(input.value || input.getAttribute('value') || defaultValue));
}
exports.createRangeTooltip = createRangeTooltip;
// 有些网课会改变 media.play 方法，所以可能不是一个 promise
function playMedia(playFunction) {
    return __awaiter(this, void 0, void 0, function* () {
        //  尝试播放
        const tryPlayMedia = () => {
            return new Promise((resolve, reject) => {
                try {
                    const playRes = playFunction();
                    if (playRes) {
                        playRes.then(resolve).catch(reject);
                    }
                    else {
                        resolve();
                    }
                }
                catch (err) {
                    reject(err);
                }
            });
        };
        try {
            yield tryPlayMedia();
            return true;
        }
        catch (err) {
            console.error(err);
            if (String(err).includes(`failed because the user didn't interact with the document first`)) {
                easy_us_1.$modal.alert({
                    content: '播放音视频失败，由于浏览器的用户隐私保护措施，如果要播放带有音量的视频，或者某些无法自动播放音视频的网站，您必须先点击一次页面上的任意位置脚本才能进行音视频的播放，后续无需重新点击。',
                    onClose: () => __awaiter(this, void 0, void 0, function* () {
                        yield tryPlayMedia();
                    })
                });
                return true;
            }
            else if (String(err).includes('The element has no supported sources')) {
                background_1.$console.error('当前视频无法播放。');
            }
            else {
                background_1.$console.error('播放视频时发生未知错误：' + String(err));
            }
            return false;
        }
    });
}
exports.playMedia = playMedia;
/**
 * 	解除复制限制功能
 * @param elements 要开启复制功能的元素
 */
function enableCopy(elements) {
    // 将页面上的所有选择方法劫持，并强制返回 true
    function hackSelect(target) {
        if (target) {
            const _original_select = target.onselectstart;
            const _original_oncopy = target.oncopy;
            const _original_onpaste = target.onpaste;
            const _original_onkeydown = target.onkeydown;
            target.onselectstart = (e) => {
                _original_select === null || _original_select === void 0 ? void 0 : _original_select.apply(target, [e]);
                e.stopPropagation();
                e.returnValue = true;
                return true;
            };
            target.oncopy = (e) => {
                _original_oncopy === null || _original_oncopy === void 0 ? void 0 : _original_oncopy.apply(target, [e]);
                e.stopPropagation();
                e.returnValue = true;
                return true;
            };
            target.onpaste = (e) => {
                _original_onpaste === null || _original_onpaste === void 0 ? void 0 : _original_onpaste.apply(target, [e]);
                e.stopPropagation();
                e.returnValue = true;
                return true;
            };
            target.onkeydown = (e) => {
                _original_onkeydown === null || _original_onkeydown === void 0 ? void 0 : _original_onkeydown.apply(target, [e]);
                e.stopPropagation();
                e.returnValue = true;
                return true;
            };
        }
    }
    for (const el of elements) {
        hackSelect(el);
    }
}
exports.enableCopy = enableCopy;
