"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeRedundantWords = exports.simplifyWorkResult = exports.createUnVisibleTextOfImage = exports.optimizationElementWithImage = exports.createWorkerControl = exports.commonWork = void 0;
const easy_us_1 = require("easy-us");
const common_1 = require("../projects/common");
const _1 = require(".");
/**
 * 通用作业考试工具方法
 */
function commonWork(script, options) {
    // 置顶当前脚本
    common_1.CommonProject.scripts.render.methods.pin(script);
    let worker;
    /**
     * 是否已经按下了开始按钮
     */
    let startBtnPressed = false;
    /**
     * 是否检查失败
     */
    let checkFailed = false;
    /** 显示答题控制按钮 */
    const createControls = () => {
        const { controlBtn, restartBtn, startBtn } = createWorkerControl({
            workerProvider: () => worker,
            onStart: () => __awaiter(this, void 0, void 0, function* () {
                startBtnPressed = true;
                checkMessage === null || checkMessage === void 0 ? void 0 : checkMessage.remove();
                start();
            }),
            onRestart: () => __awaiter(this, void 0, void 0, function* () {
                var _a;
                worker === null || worker === void 0 ? void 0 : worker.emit('close');
                yield ((_a = options.onRestart) === null || _a === void 0 ? void 0 : _a.call(options));
                start();
            })
        });
        startBtn.style.flex = '1';
        startBtn.style.padding = '4px';
        restartBtn.style.flex = '1';
        restartBtn.style.padding = '4px';
        controlBtn.style.flex = '1';
        controlBtn.style.padding = '4px';
        const container = (0, easy_us_1.h)('div', { style: { marginTop: '12px', display: 'flex' } }, (worker === null || worker === void 0 ? void 0 : worker.isRunning) ? [controlBtn, restartBtn] : [startBtn]);
        return { container, startBtn, restartBtn, controlBtn };
    };
    const workResultPanel = () => common_1.CommonProject.scripts.workResults.methods.createWorkResultsPanel();
    script.on('render', () => {
        var _a, _b;
        let gotoSettingsBtnContainer = '';
        if (checkFailed) {
            const gotoSettingsBtn = easy_us_1.$ui.button('👉 前往设置题库配置', {
                className: 'base-style-button',
                style: { flex: '1', padding: '4px' }
            });
            gotoSettingsBtn.style.flex = '1';
            gotoSettingsBtn.style.padding = '4px';
            gotoSettingsBtn.onclick = () => {
                common_1.CommonProject.scripts.render.methods.pin(common_1.CommonProject.scripts.settings);
            };
            gotoSettingsBtnContainer = (0, easy_us_1.h)('div', { style: { display: 'flex' } }, [gotoSettingsBtn]);
        }
        (_b = (_a = script.panel) === null || _a === void 0 ? void 0 : _a.body) === null || _b === void 0 ? void 0 : _b.replaceChildren((0, easy_us_1.h)('div', { style: { marginTop: '12px' } }, [
            gotoSettingsBtnContainer,
            createControls().container,
            workResultPanel()
        ]));
    });
    const workOptions = common_1.CommonProject.scripts.settings.methods.getWorkOptions();
    /**
     * 检查题库是否配置，并询问是否开始答题
     */
    let checkMessage = (0, _1.workPreCheckMessage)(Object.assign({ onrun: () => startBtnPressed === false && start(), onclose: (_, closedMsg) => (checkMessage = closedMsg), onNoAnswererWrappers: () => {
            checkFailed = true;
        } }, workOptions));
    const start = () => __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c, _d;
        yield ((_a = options.beforeRunning) === null || _a === void 0 ? void 0 : _a.call(options));
        worker = options.workerProvider(workOptions);
        if (worker) {
            (_b = options.onWorkerCreated) === null || _b === void 0 ? void 0 : _b.call(options, worker);
        }
        const { container, controlBtn } = createControls();
        // 更新状态
        (_d = (_c = script.panel) === null || _c === void 0 ? void 0 : _c.body) === null || _d === void 0 ? void 0 : _d.replaceChildren(container, workResultPanel());
        worker === null || worker === void 0 ? void 0 : worker.once('done', () => {
            controlBtn.disabled = true;
        });
    });
}
exports.commonWork = commonWork;
/**
 * 答题控制
 */
function createWorkerControl(options) {
    let stop = false;
    let stopMessage;
    const startBtn = easy_us_1.$ui.button('▶️开始答题');
    const restartBtn = easy_us_1.$ui.button('🔃重新答题');
    const controlBtn = easy_us_1.$ui.button('⏸暂停');
    startBtn.onclick = () => {
        startBtn.remove();
        options.onStart();
    };
    restartBtn.onclick = () => {
        // 重新答题时，清除暂停提示
        stopMessage === null || stopMessage === void 0 ? void 0 : stopMessage.remove();
        options.onRestart();
    };
    controlBtn.onclick = () => {
        var _a;
        stop = !stop;
        const worker = options.workerProvider();
        (_a = worker === null || worker === void 0 ? void 0 : worker.emit) === null || _a === void 0 ? void 0 : _a.call(worker, stop ? 'stop' : 'continuate');
        controlBtn.value = stop ? '▶️继续' : '⏸️暂停';
        if (stop) {
            stopMessage = easy_us_1.$message.warn({ duration: 0, content: '暂停中...' });
        }
        else {
            stopMessage === null || stopMessage === void 0 ? void 0 : stopMessage.remove();
        }
    };
    return { startBtn, restartBtn, controlBtn };
}
exports.createWorkerControl = createWorkerControl;
/**
 * 图片识别，将图片链接追加到 text 中
 */
function optimizationElementWithImage(root) {
    if (root) {
        for (const img of Array.from(root.querySelectorAll('img'))) {
            const src = document.createElement('span');
            src.innerText = img.src;
            // 隐藏图片，但不影响 innerText 的获取
            src.style.fontSize = '0px';
            img.after(src);
        }
    }
    return root;
}
exports.optimizationElementWithImage = optimizationElementWithImage;
/**
 * 创建一个不可见的文本节点，追加到图片后面，便于文本获取
 */
function createUnVisibleTextOfImage(img) {
    const src = document.createElement('span');
    src.innerText = img.src;
    // 隐藏图片，但不影响 innerText 的获取
    src.style.fontSize = '0px';
    img.after(src);
}
exports.createUnVisibleTextOfImage = createUnVisibleTextOfImage;
/** 将 {@link WorkResult} 转换成 {@link SimplifyWorkResult} */
function simplifyWorkResult(results, 
/**
 * 标题处理方法
 * 在答题时使用相同的处理方法，可以使答题结果显示的题目与搜题的题目保持一致
 */
titleTransform) {
    var _a, _b, _c, _d, _e;
    const res = [];
    let i = 0;
    for (const wr of results) {
        res.push({
            requested: wr.requested,
            resolved: wr.resolved,
            error: wr.error,
            question: (titleTransform === null || titleTransform === void 0 ? void 0 : titleTransform(((_a = wr.ctx) === null || _a === void 0 ? void 0 : _a.elements.title) || [], i)) || ((_c = (_b = wr.ctx) === null || _b === void 0 ? void 0 : _b.elements.title) === null || _c === void 0 ? void 0 : _c.join(',')) || '',
            finish: (_d = wr.result) === null || _d === void 0 ? void 0 : _d.finish,
            searchInfos: ((_e = wr.ctx) === null || _e === void 0 ? void 0 : _e.searchInfos.map((sr) => ({
                error: sr.error,
                name: sr.name,
                homepage: sr.homepage,
                results: sr.results.map((ans) => [ans.question, ans.answer])
            }))) || []
        });
        i++;
    }
    return res;
}
exports.simplifyWorkResult = simplifyWorkResult;
/**
 * 从题目中移除指定的冗余词
 */
function removeRedundantWords(str, words) {
    for (const word of words) {
        str = str.replace(word, '');
    }
    return str;
}
exports.removeRedundantWords = removeRedundantWords;
